################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zframe(object):
    """
    working with single message frames
    """

    def __init__(self, data, size):
        """
        Create a new frame. If size is not null, allocates the frame data
        to the specified size. If additionally, data is not null, copies
        size octets from the specified data into the frame body.
        """
        p = utils.lib.zframe_new(data, size)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zframe_destroy_py)

    @staticmethod
    def new_empty():
        """
        Create an empty (zero-sized) frame
        """
        return utils.lib.zframe_new_empty()

    @staticmethod
    def from_py(string):
        """
        Create a frame with a specified string content.
        """
        return utils.lib.zframe_from(utils.to_bytes(string))

    @staticmethod
    def frommem(data, size, destructor, hint):
        """
        Create a new frame from memory. Take ownership of the memory and calling the destructor
        on destroy.
        """
        return utils.lib.zframe_frommem(data, size, destructor, hint._p)

    @staticmethod
    def recv(source):
        """
        Receive frame from socket, returns zframe_t object or NULL if the recv
        was interrupted. Does a blocking recv, if you want to not block then use
        zpoller or zloop.
        """
        return utils.lib.zframe_recv(source._p)

    @staticmethod
    def send(self_p, dest, flags):
        """
        Send a frame to a socket, destroy frame after sending.
        Return -1 on error, 0 on success.
        """
        return utils.lib.zframe_send(utils.ffi.new("zframe_t **", self_p._p), dest._p, flags)

    def size(self):
        """
        Return number of bytes in frame data
        """
        return utils.lib.zframe_size(self._p)

    def data(self):
        """
        Return address of frame data
        """
        return utils.lib.zframe_data(self._p)

    def meta(self, property):
        """
        Return meta data property for frame
        The caller shall not modify or free the returned value, which shall be
        owned by the message.
        """
        return utils.lib.zframe_meta(self._p, utils.to_bytes(property))

    def dup(self):
        """
        Create a new frame that duplicates an existing frame. If frame is null,
        or memory was exhausted, returns null.
        """
        return utils.lib.zframe_dup(self._p)

    def strhex(self):
        """
        Return frame data encoded as printable hex string, useful for 0MQ UUIDs.
        Caller must free string when finished with it.
        """
        return utils.lib.zframe_strhex(self._p)

    def strdup(self):
        """
        Return frame data copied into freshly allocated string
        Caller must free string when finished with it.
        """
        return utils.lib.zframe_strdup(self._p)

    def streq(self, string):
        """
        Return TRUE if frame body is equal to string, excluding terminator
        """
        return utils.lib.zframe_streq(self._p, utils.to_bytes(string))

    def more(self):
        """
        Return frame MORE indicator (1 or 0), set when reading frame from socket
        or by the zframe_set_more() method
        """
        return utils.lib.zframe_more(self._p)

    def set_more(self, more):
        """
        Set frame MORE indicator (1 or 0). Note this is NOT used when sending
        frame to socket, you have to specify flag explicitly.
        """
        utils.lib.zframe_set_more(self._p, more)

    def routing_id(self):
        """
        Return frame routing ID, if the frame came from a ZMQ_SERVER socket.
        Else returns zero.
        """
        return utils.lib.zframe_routing_id(self._p)

    def set_routing_id(self, routing_id):
        """
        Set routing ID on frame. This is used if/when the frame is sent to a
        ZMQ_SERVER socket.
        """
        utils.lib.zframe_set_routing_id(self._p, routing_id)

    def group(self):
        """
        Return frame group of radio-dish pattern.
        """
        return utils.lib.zframe_group(self._p)

    def set_group(self, group):
        """
        Set group on frame. This is used if/when the frame is sent to a
        ZMQ_RADIO socket.
        Return -1 on error, 0 on success.
        """
        return utils.lib.zframe_set_group(self._p, utils.to_bytes(group))

    def eq(self, other):
        """
        Return TRUE if two frames have identical size and data
        If either frame is NULL, equality is always false.
        """
        return utils.lib.zframe_eq(self._p, other._p)

    def reset(self, data, size):
        """
        Set new contents for frame
        """
        utils.lib.zframe_reset(self._p, data, size)

    def print_py(self, prefix):
        """
        Send message to zsys log sink (may be stdout, or system facility as
        configured by zsys_set_logstream). Prefix shows before frame, if not null.
        Long messages are truncated.
        """
        utils.lib.zframe_print(self._p, utils.to_bytes(prefix))

    def print_n(self, prefix, length):
        """
        Send message to zsys log sink (may be stdout, or system facility as
        configured by zsys_set_logstream). Prefix shows before frame, if not null.
        Message length is specified; no truncation unless length is zero.
        Backwards compatible with zframe_print when length is zero.
        """
        utils.lib.zframe_print_n(self._p, utils.to_bytes(prefix), length)

    @staticmethod
    def is_py(self):
        """
        Probe the supplied object, and report if it looks like a zframe_t.
        """
        return utils.lib.zframe_is(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zframe_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from setuptools import setup

setup(
    name = "czmq_cffi",
    version = "4.2.1",
    license = "mpl-2.0",
    description = """Python cffi bindings of: the high-level c binding for 0mq""",
    packages = ["czmq_cffi", ],
    setup_requires=["cffi"],
    cffi_modules=[
           "czmq_cffi/build.py:ffibuilder",
           "czmq_cffi/build.py:ffidestructorbuilder"
    ],
    install_requires=["cffi"],
)
################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zsys(object):
    """

    """

    @staticmethod
    def init():
        """
        Initialize CZMQ zsys layer; this happens automatically when you create
        a socket or an actor; however this call lets you force initialization
        earlier, so e.g. logging is properly set-up before you start working.
        Not threadsafe, so call only from main thread. Safe to call multiple
        times. Returns global CZMQ context.
        """
        return utils.lib.zsys_init()

    @staticmethod
    def shutdown():
        """
        Optionally shut down the CZMQ zsys layer; this normally happens automatically
        when the process exits; however this call lets you force a shutdown
        earlier, avoiding any potential problems with atexit() ordering, especially
        with Windows dlls.
        """
        utils.lib.zsys_shutdown()

    @staticmethod
    def socket(type, filename, line_nbr):
        """
        Get a new ZMQ socket, automagically creating a ZMQ context if this is
        the first time. Caller is responsible for destroying the ZMQ socket
        before process exits, to avoid a ZMQ deadlock. Note: you should not use
        this method in CZMQ apps, use zsock_new() instead.
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        return utils.lib.zsys_socket(type, utils.to_bytes(filename), line_nbr)

    @staticmethod
    def close(handle, filename, line_nbr):
        """
        Destroy/close a ZMQ socket. You should call this for every socket you
        create using zsys_socket().
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        return utils.lib.zsys_close(handle._p, utils.to_bytes(filename), line_nbr)

    @staticmethod
    def sockname(socktype):
        """
        Return ZMQ socket name for socket type
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        return utils.lib.zsys_sockname(socktype)

    @staticmethod
    def create_pipe(backend_p):
        """
        Create a pipe, which consists of two PAIR sockets connected over inproc.
        The pipe is configured to use the zsys_pipehwm setting. Returns the
        frontend socket successful, NULL if failed.
        """
        return utils.lib.zsys_create_pipe(utils.ffi.new("zsock_t **", backend_p._p))

    @staticmethod
    def handler_set(handler_fn):
        """
        Set interrupt handler; this saves the default handlers so that a
        zsys_handler_reset () can restore them. If you call this multiple times
        then the last handler will take affect. If handler_fn is NULL, disables
        default SIGINT/SIGTERM handling in CZMQ.
        """
        utils.lib.zsys_handler_set(handler_fn)

    @staticmethod
    def handler_reset():
        """
        Reset interrupt handler, call this at exit if needed
        """
        utils.lib.zsys_handler_reset()

    @staticmethod
    def catch_interrupts():
        """
        Set default interrupt handler, so Ctrl-C or SIGTERM will set
        zsys_interrupted. Idempotent; safe to call multiple times.
        Can be suppressed by ZSYS_SIGHANDLER=false
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        utils.lib.zsys_catch_interrupts()

    @staticmethod
    def is_interrupted():
        """
        Check if default interrupt handler of Ctrl-C or SIGTERM was called.
        Does not work if ZSYS_SIGHANDLER is false and code does not call
        set interrupted on signal.
        """
        return utils.lib.zsys_is_interrupted()

    @staticmethod
    def set_interrupted():
        """
        Set interrupted flag. This is done by default signal handler, however
        this can be handy for language bindings or cases without default
        signal handler.
        """
        utils.lib.zsys_set_interrupted()

    @staticmethod
    def file_exists(filename):
        """
        Return 1 if file exists, else zero
        """
        return utils.lib.zsys_file_exists(utils.to_bytes(filename))

    @staticmethod
    def file_modified(filename):
        """
        Return file modification time. Returns 0 if the file does not exist.
        """
        return utils.lib.zsys_file_modified(utils.to_bytes(filename))

    @staticmethod
    def file_mode(filename):
        """
        Return file mode; provides at least support for the POSIX S_ISREG(m)
        and S_ISDIR(m) macros and the S_IRUSR and S_IWUSR bits, on all boxes.
        Returns a mode_t cast to int, or -1 in case of error.
        """
        return utils.lib.zsys_file_mode(utils.to_bytes(filename))

    @staticmethod
    def file_delete(filename):
        """
        Delete file. Does not complain if the file is absent
        """
        return utils.lib.zsys_file_delete(utils.to_bytes(filename))

    @staticmethod
    def file_stable(filename):
        """
        Check if file is 'stable'
        """
        return utils.lib.zsys_file_stable(utils.to_bytes(filename))

    @staticmethod
    def dir_create(pathname, *pathname_args):
        """
        Create a file path if it doesn't exist. The file path is treated as
        printf format.
        """
        return utils.lib.zsys_dir_create(utils.to_bytes(pathname), *pathname_args)

    @staticmethod
    def dir_delete(pathname, *pathname_args):
        """
        Remove a file path if empty; the pathname is treated as printf format.
        """
        return utils.lib.zsys_dir_delete(utils.to_bytes(pathname), *pathname_args)

    @staticmethod
    def dir_change(pathname):
        """
        Move to a specified working directory. Returns 0 if OK, -1 if this failed.
        """
        return utils.lib.zsys_dir_change(utils.to_bytes(pathname))

    @staticmethod
    def file_mode_private():
        """
        Set private file creation mode; all files created from here will be
        readable/writable by the owner only.
        """
        utils.lib.zsys_file_mode_private()

    @staticmethod
    def file_mode_default():
        """
        Reset default file creation mode; all files created from here will use
        process file mode defaults.
        """
        utils.lib.zsys_file_mode_default()

    @staticmethod
    def version(major, minor, patch):
        """
        Return the CZMQ version for run-time API detection; returns version
        number into provided fields, providing reference isn't null in each case.
        """
        utils.lib.zsys_version(major, minor, patch)

    @staticmethod
    def sprintf_hint(hint, format, *format_args):
        """
        Format a string using printf formatting, returning a freshly allocated
        buffer. If there was insufficient memory, returns NULL. Free the returned
        string using zstr_free(). The hinted version allows to optimize by using
        a larger starting buffer size (known to/assumed by the developer) and so
        avoid reallocations.
        """
        return utils.lib.zsys_sprintf_hint(hint, utils.to_bytes(format), *format_args)

    @staticmethod
    def sprintf(format, *format_args):
        """
        Format a string using printf formatting, returning a freshly allocated
        buffer. If there was insufficient memory, returns NULL. Free the returned
        string using zstr_free().
        """
        return utils.lib.zsys_sprintf(utils.to_bytes(format), *format_args)

    @staticmethod
    def vprintf(format, argptr):
        """
        Format a string with a va_list argument, returning a freshly allocated
        buffer. If there was insufficient memory, returns NULL. Free the returned
        string using zstr_free().
        """
        return utils.lib.zsys_vprintf(utils.to_bytes(format), argptr._p)

    @staticmethod
    def udp_new(routable):
        """
        Create UDP beacon socket; if the routable option is true, uses
        multicast (not yet implemented), else uses broadcast. This method
        and related ones might _eventually_ be moved to a zudp class.
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        return utils.lib.zsys_udp_new(routable)

    @staticmethod
    def udp_close(handle):
        """
        Close a UDP socket
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        return utils.lib.zsys_udp_close(handle._p)

    @staticmethod
    def udp_send(udpsock, frame, address, addrlen):
        """
        Send zframe to UDP socket, return -1 if sending failed due to
        interface having disappeared (happens easily with WiFi)
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        return utils.lib.zsys_udp_send(udpsock._p, frame._p, address._p, addrlen)

    @staticmethod
    def udp_recv(udpsock, peername, peerlen):
        """
        Receive zframe from UDP socket, and set address of peer that sent it
        The peername must be a char [INET_ADDRSTRLEN] array if IPv6 is disabled or
        NI_MAXHOST if it's enabled. Returns NULL when failing to get peer address.
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        return utils.lib.zsys_udp_recv(udpsock._p, utils.to_bytes(peername), peerlen)

    @staticmethod
    def socket_error(reason):
        """
        Handle an I/O error on some socket operation; will report and die on
        fatal errors, and continue silently on "try again" errors.
        *** This is for CZMQ internal use only and may change arbitrarily ***
        """
        utils.lib.zsys_socket_error(utils.to_bytes(reason))

    @staticmethod
    def hostname():
        """
        Return current host name, for use in public tcp:// endpoints. Caller gets
        a freshly allocated string, should free it using zstr_free(). If the host
        name is not resolvable, returns NULL.
        """
        return utils.lib.zsys_hostname()

    @staticmethod
    def daemonize(workdir):
        """
        Move the current process into the background. The precise effect depends
        on the operating system. On POSIX boxes, moves to a specified working
        directory (if specified), closes all file handles, reopens stdin, stdout,
        and stderr to the null device, and sets the process to ignore SIGHUP. On
        Windows, does nothing. Returns 0 if OK, -1 if there was an error.
        """
        return utils.lib.zsys_daemonize(utils.to_bytes(workdir))

    @staticmethod
    def run_as(lockfile, group, user):
        """
        Drop the process ID into the lockfile, with exclusive lock, and switch
        the process to the specified group and/or user. Any of the arguments
        may be null, indicating a no-op. Returns 0 on success, -1 on failure.
        Note if you combine this with zsys_daemonize, run after, not before
        that method, or the lockfile will hold the wrong process ID.
        """
        return utils.lib.zsys_run_as(utils.to_bytes(lockfile), utils.to_bytes(group), utils.to_bytes(user))

    @staticmethod
    def has_curve():
        """
        Returns true if the underlying libzmq supports CURVE security.
        Uses a heuristic probe according to the version of libzmq being used.
        """
        return utils.lib.zsys_has_curve()

    @staticmethod
    def set_io_threads(io_threads):
        """
        Configure the number of I/O threads that ZeroMQ will use. A good
        rule of thumb is one thread per gigabit of traffic in or out. The
        default is 1, sufficient for most applications. If the environment
        variable ZSYS_IO_THREADS is defined, that provides the default.
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_set_io_threads(io_threads)

    @staticmethod
    def set_thread_sched_policy(policy):
        """
        Configure the scheduling policy of the ZMQ context thread pool.
        Not available on Windows. See the sched_setscheduler man page or sched.h
        for more information. If the environment variable ZSYS_THREAD_SCHED_POLICY
        is defined, that provides the default.
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_set_thread_sched_policy(policy)

    @staticmethod
    def set_thread_priority(priority):
        """
        Configure the scheduling priority of the ZMQ context thread pool.
        Not available on Windows. See the sched_setscheduler man page or sched.h
        for more information. If the environment variable ZSYS_THREAD_PRIORITY is
        defined, that provides the default.
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_set_thread_priority(priority)

    @staticmethod
    def set_thread_name_prefix(prefix):
        """
        Configure the numeric prefix to each thread created for the internal
        context's thread pool. This option is only supported on Linux.
        If the environment variable ZSYS_THREAD_NAME_PREFIX is defined, that
        provides the default.
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_set_thread_name_prefix(prefix)

    @staticmethod
    def thread_name_prefix():
        """
        Return thread name prefix.
        """
        return utils.lib.zsys_thread_name_prefix()

    @staticmethod
    def set_thread_name_prefix_str(prefix):
        """
        Configure the numeric prefix to each thread created for the internal
        context's thread pool. This option is only supported on Linux.
        If the environment variable ZSYS_THREAD_NAME_PREFIX_STR is defined, that
        provides the default.
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_set_thread_name_prefix_str(utils.to_bytes(prefix))

    @staticmethod
    def thread_name_prefix_str():
        """
        Return thread name prefix.
        """
        return utils.lib.zsys_thread_name_prefix_str()

    @staticmethod
    def thread_affinity_cpu_add(cpu):
        """
        Adds a specific CPU to the affinity list of the ZMQ context thread pool.
        This option is only supported on Linux.
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_thread_affinity_cpu_add(cpu)

    @staticmethod
    def thread_affinity_cpu_remove(cpu):
        """
        Removes a specific CPU to the affinity list of the ZMQ context thread pool.
        This option is only supported on Linux.
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_thread_affinity_cpu_remove(cpu)

    @staticmethod
    def set_max_sockets(max_sockets):
        """
        Configure the number of sockets that ZeroMQ will allow. The default
        is 1024. The actual limit depends on the system, and you can query it
        by using zsys_socket_limit (). A value of zero means "maximum".
        Note that this method is valid only before any socket is created.
        """
        utils.lib.zsys_set_max_sockets(max_sockets)

    @staticmethod
    def socket_limit():
        """
        Return maximum number of ZeroMQ sockets that the system will support.
        """
        return utils.lib.zsys_socket_limit()

    @staticmethod
    def set_max_msgsz(max_msgsz):
        """
        Configure the maximum allowed size of a message sent.
        The default is INT_MAX.
        """
        utils.lib.zsys_set_max_msgsz(max_msgsz)

    @staticmethod
    def max_msgsz():
        """
        Return maximum message size.
        """
        return utils.lib.zsys_max_msgsz()

    @staticmethod
    def set_zero_copy_recv(zero_copy):
        """
        Configure whether to use zero copy strategy in libzmq. If the environment
        variable ZSYS_ZERO_COPY_RECV is defined, that provides the default.
        Otherwise the default is 1.
        """
        utils.lib.zsys_set_zero_copy_recv(zero_copy)

    @staticmethod
    def zero_copy_recv():
        """
        Return ZMQ_ZERO_COPY_RECV option.
        """
        return utils.lib.zsys_zero_copy_recv()

    @staticmethod
    def set_file_stable_age_msec(file_stable_age_msec):
        """
        Configure the threshold value of filesystem object age per st_mtime
        that should elapse until we consider that object "stable" at the
        current zclock_time() moment.
        The default is S_DEFAULT_ZSYS_FILE_STABLE_AGE_MSEC defined in zsys.c
        which generally depends on host OS, with fallback value of 5000.
        """
        utils.lib.zsys_set_file_stable_age_msec(file_stable_age_msec._p)

    @staticmethod
    def file_stable_age_msec():
        """
        Return current threshold value of file stable age in msec.
        This can be used in code that chooses to wait for this timeout
        before testing if a filesystem object is "stable" or not.
        """
        return utils.lib.zsys_file_stable_age_msec()

    @staticmethod
    def set_linger(linger):
        """
        Configure the default linger timeout in msecs for new zsock instances.
        You can also set this separately on each zsock_t instance. The default
        linger time is zero, i.e. any pending messages will be dropped. If the
        environment variable ZSYS_LINGER is defined, that provides the default.
        Note that process exit will typically be delayed by the linger time.
        """
        utils.lib.zsys_set_linger(linger)

    @staticmethod
    def set_sndhwm(sndhwm):
        """
        Configure the default outgoing pipe limit (HWM) for new zsock instances.
        You can also set this separately on each zsock_t instance. The default
        HWM is 1,000, on all versions of ZeroMQ. If the environment variable
        ZSYS_SNDHWM is defined, that provides the default. Note that a value of
        zero means no limit, i.e. infinite memory consumption.
        """
        utils.lib.zsys_set_sndhwm(sndhwm)

    @staticmethod
    def set_rcvhwm(rcvhwm):
        """
        Configure the default incoming pipe limit (HWM) for new zsock instances.
        You can also set this separately on each zsock_t instance. The default
        HWM is 1,000, on all versions of ZeroMQ. If the environment variable
        ZSYS_RCVHWM is defined, that provides the default. Note that a value of
        zero means no limit, i.e. infinite memory consumption.
        """
        utils.lib.zsys_set_rcvhwm(rcvhwm)

    @staticmethod
    def set_pipehwm(pipehwm):
        """
        Configure the default HWM for zactor internal pipes; this is set on both
        ends of the pipe, for outgoing messages only (sndhwm). The default HWM is
        1,000, on all versions of ZeroMQ. If the environment var ZSYS_ACTORHWM is
        defined, that provides the default. Note that a value of zero means no
        limit, i.e. infinite memory consumption.
        """
        utils.lib.zsys_set_pipehwm(pipehwm)

    @staticmethod
    def pipehwm():
        """
        Return the HWM for zactor internal pipes.
        """
        return utils.lib.zsys_pipehwm()

    @staticmethod
    def set_ipv6(ipv6):
        """
        Configure use of IPv6 for new zsock instances. By default sockets accept
        and make only IPv4 connections. When you enable IPv6, sockets will accept
        and connect to both IPv4 and IPv6 peers. You can override the setting on
        each zsock_t instance. The default is IPv4 only (ipv6 set to 0). If the
        environment variable ZSYS_IPV6 is defined (as 1 or 0), this provides the
        default. Note: has no effect on ZMQ v2.
        """
        utils.lib.zsys_set_ipv6(ipv6)

    @staticmethod
    def ipv6():
        """
        Return use of IPv6 for zsock instances.
        """
        return utils.lib.zsys_ipv6()

    @staticmethod
    def ipv6_available():
        """
        Test if ipv6 is available on the system. Return true if available.
        The only way to reliably check is to actually open a socket and
        try to bind it. (ported from libzmq)
        """
        return utils.lib.zsys_ipv6_available()

    @staticmethod
    def set_interface(value):
        """
        Set network interface name to use for broadcasts, particularly zbeacon.
        This lets the interface be configured for test environments where required.
        For example, on Mac OS X, zbeacon cannot bind to 255.255.255.255 which is
        the default when there is no specified interface. If the environment
        variable ZSYS_INTERFACE is set, use that as the default interface name.
        Setting the interface to "*" means "use all available interfaces".
        """
        utils.lib.zsys_set_interface(utils.to_bytes(value))

    @staticmethod
    def interface():
        """
        Return network interface to use for broadcasts, or "" if none was set.
        """
        return utils.lib.zsys_interface()

    @staticmethod
    def set_ipv6_address(value):
        """
        Set IPv6 address to use zbeacon socket, particularly for receiving zbeacon.
        This needs to be set IPv6 is enabled as IPv6 can have multiple addresses
        on a given interface. If the environment variable ZSYS_IPV6_ADDRESS is set,
        use that as the default IPv6 address.
        """
        utils.lib.zsys_set_ipv6_address(utils.to_bytes(value))

    @staticmethod
    def ipv6_address():
        """
        Return IPv6 address to use for zbeacon reception, or "" if none was set.
        """
        return utils.lib.zsys_ipv6_address()

    @staticmethod
    def set_ipv6_mcast_address(value):
        """
        Set IPv6 milticast address to use for sending zbeacon messages. This needs
        to be set if IPv6 is enabled. If the environment variable
        ZSYS_IPV6_MCAST_ADDRESS is set, use that as the default IPv6 multicast
        address.
        """
        utils.lib.zsys_set_ipv6_mcast_address(utils.to_bytes(value))

    @staticmethod
    def ipv6_mcast_address():
        """
        Return IPv6 multicast address to use for sending zbeacon, or "" if none was
        set.
        """
        return utils.lib.zsys_ipv6_mcast_address()

    @staticmethod
    def set_ipv4_mcast_address(value):
        """
        Set IPv4 multicast address to use for sending zbeacon messages. By default
        IPv4 multicast is NOT used. If the environment variable
        ZSYS_IPV4_MCAST_ADDRESS is set, use that as the default IPv4 multicast
        address. Calling this function or setting ZSYS_IPV4_MCAST_ADDRESS
        will enable IPv4 zbeacon messages.
        """
        utils.lib.zsys_set_ipv4_mcast_address(utils.to_bytes(value))

    @staticmethod
    def ipv4_mcast_address():
        """
        Return IPv4 multicast address to use for sending zbeacon, or NULL if none was
        set.
        """
        return utils.lib.zsys_ipv4_mcast_address()

    @staticmethod
    def set_mcast_ttl(value):
        """
        Set multicast TTL default is 1
        """
        utils.lib.zsys_set_mcast_ttl(value)

    @staticmethod
    def mcast_ttl():
        """
        Get multicast TTL
        """
        return utils.lib.zsys_mcast_ttl()

    @staticmethod
    def set_auto_use_fd(auto_use_fd):
        """
        Configure the automatic use of pre-allocated FDs when creating new sockets.
        If 0 (default), nothing will happen. Else, when a new socket is bound, the
        system API will be used to check if an existing pre-allocated FD with a
        matching port (if TCP) or path (if IPC) exists, and if it does it will be
        set via the ZMQ_USE_FD socket option so that the library will use it
        instead of creating a new socket.
        """
        utils.lib.zsys_set_auto_use_fd(auto_use_fd)

    @staticmethod
    def auto_use_fd():
        """
        Return use of automatic pre-allocated FDs for zsock instances.
        """
        return utils.lib.zsys_auto_use_fd()

    @staticmethod
    def zprintf(format, args):
        """
        Print formatted string. Format is specified by variable names
        in Python-like format style

        "%(KEY)s=%(VALUE)s", KEY=key, VALUE=value
        become
        "key=value"

        Returns freshly allocated string or NULL in a case of error.
        Not enough memory, invalid format specifier, name not in args
        """
        return utils.lib.zsys_zprintf(utils.to_bytes(format), args._p)

    @staticmethod
    def zprintf_error(format, args):
        """
        Return error string for given format/args combination.
        """
        return utils.lib.zsys_zprintf_error(utils.to_bytes(format), args._p)

    @staticmethod
    def zplprintf(format, args):
        """
        Print formatted string. Format is specified by variable names
        in Python-like format style

        "%(KEY)s=%(VALUE)s", KEY=key, VALUE=value
        become
        "key=value"

        Returns freshly allocated string or NULL in a case of error.
        Not enough memory, invalid format specifier, name not in args
        """
        return utils.lib.zsys_zplprintf(utils.to_bytes(format), args._p)

    @staticmethod
    def zplprintf_error(format, args):
        """
        Return error string for given format/args combination.
        """
        return utils.lib.zsys_zplprintf_error(utils.to_bytes(format), args._p)

    @staticmethod
    def set_logident(value):
        """
        Set log identity, which is a string that prefixes all log messages sent
        by this process. The log identity defaults to the environment variable
        ZSYS_LOGIDENT, if that is set.
        """
        utils.lib.zsys_set_logident(utils.to_bytes(value))

    @staticmethod
    def set_logstream(stream):
        """
        Set stream to receive log traffic. By default, log traffic is sent to
        stdout. If you set the stream to NULL, no stream will receive the log
        traffic (it may still be sent to the system facility).
        """
        utils.lib.zsys_set_logstream(stream)

    @staticmethod
    def set_logsender(endpoint):
        """
        Sends log output to a PUB socket bound to the specified endpoint. To
        collect such log output, create a SUB socket, subscribe to the traffic
        you care about, and connect to the endpoint. Log traffic is sent as a
        single string frame, in the same format as when sent to stdout. The
        log system supports a single sender; multiple calls to this method will
        bind the same sender to multiple endpoints. To disable the sender, call
        this method with a null argument.
        """
        utils.lib.zsys_set_logsender(utils.to_bytes(endpoint))

    @staticmethod
    def set_logsystem(logsystem):
        """
        Enable or disable logging to the system facility (syslog on POSIX boxes,
        event log on Windows). By default this is disabled.
        """
        utils.lib.zsys_set_logsystem(logsystem)

    @staticmethod
    def error(format, *format_args):
        """
        Log error condition - highest priority
        """
        utils.lib.zsys_error(utils.to_bytes(format), *format_args)

    @staticmethod
    def warning(format, *format_args):
        """
        Log warning condition - high priority
        """
        utils.lib.zsys_warning(utils.to_bytes(format), *format_args)

    @staticmethod
    def notice(format, *format_args):
        """
        Log normal, but significant, condition - normal priority
        """
        utils.lib.zsys_notice(utils.to_bytes(format), *format_args)

    @staticmethod
    def info(format, *format_args):
        """
        Log informational message - low priority
        """
        utils.lib.zsys_info(utils.to_bytes(format), *format_args)

    @staticmethod
    def debug(format, *format_args):
        """
        Log debug-level message - lowest priority
        """
        utils.lib.zsys_debug(utils.to_bytes(format), *format_args)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zsys_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class ZhttpServer(object):
    """
    Simple http server.
To start handling requests:
1. Create a dealer socket
2. Connect the socket to the server backend address provided in the options.
3. Create a zhttp_request.
4. Call zhttp_request_recv to accept a new request.
5. Call zhttp_response_send to send a response.

You can connect as many dealers as you want.
The Server is using simple dealer socket to route the requests.

NOTE: when using libmicrohttpd << 0.9.34 the application might experience
high CPU usage due to the lack of MHD_suspend_connection and
MHD_resume_connection APIs. It is recommended to use this class only with
libmicrohttpd at least 0.9.34 in production.
    """

    def __init__(self, options):
        """
        Create a new http server
        """
        p = utils.lib.zhttp_server_new(options._p)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zhttp_server_destroy_py)

    def port(self):
        """
        Return the port the server is listening on.
        """
        return utils.lib.zhttp_server_port(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhttp_server_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class ZhttpClient(object):
    """
    Http client, allowing multiple requests simultaneously and integrate easily with zpoller.
Use zhttp_request class to create and send the request.
Use zhttp_response class to receive the response.
    """

    def __init__(self, verbose):
        """
        Create a new http client
        """
        p = utils.lib.zhttp_client_new(verbose)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zhttp_client_destroy_py)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhttp_client_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zargs(object):
    """
    Platform independent command line argument parsing helpers

There are two kind of elements provided by this class
Named parameters, accessed by param_get and param_has methods
  * --named-parameter
  * --parameter with_value
  * -a val
Positional arguments, accessed by zargs_first, zargs_next

It DOES:
* provide easy to use CLASS compatible API for accessing argv
* is platform independent
* provide getopt_long style -- argument, which delimits parameters from arguments
* makes parameters position independent

It does NOT
* change argv
* provide a "declarative" way to define command line interface

In future it SHALL
* hide several formats of command line to one (-Idir, --include=dir,
  --include dir are the same from API pov)
    """

    def __init__(self, argc, argv):
        """
        Create a new zargs from command line arguments.
        """
        p = utils.lib.zargs_new(argc, utils.to_bytes(argv))
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zargs_destroy_py)

    def progname(self):
        """
        Return program name (argv[0])
        """
        return utils.lib.zargs_progname(self._p)

    def arguments(self):
        """
        Return number of positional arguments
        """
        return utils.lib.zargs_arguments(self._p)

    def first(self):
        """
        Return first positional argument or NULL
        """
        return utils.lib.zargs_first(self._p)

    def next(self):
        """
        Return next positional argument or NULL
        """
        return utils.lib.zargs_next(self._p)

    def param_first(self):
        """
        Return first named parameter value, or NULL if there are no named
        parameters, or value for which zargs_param_empty (arg) returns true.
        """
        return utils.lib.zargs_param_first(self._p)

    def param_next(self):
        """
        Return next named parameter value, or NULL if there are no named
        parameters, or value for which zargs_param_empty (arg) returns true.
        """
        return utils.lib.zargs_param_next(self._p)

    def param_name(self):
        """
        Return current parameter name, or NULL if there are no named parameters.
        """
        return utils.lib.zargs_param_name(self._p)

    def get(self, name):
        """
        Return value of named parameter or NULL is it has no value (or was not specified)
        """
        return utils.lib.zargs_get(self._p, utils.to_bytes(name))

    def getx(self, name, *name_args):
        """
        Return value of one of parameter(s) or NULL is it has no value (or was not specified)
        """
        return utils.lib.zargs_getx(self._p, utils.to_bytes(name), *name_args)

    def has(self, name):
        """
        Returns true if named parameter was specified on command line
        """
        return utils.lib.zargs_has(self._p, utils.to_bytes(name))

    def hasx(self, name, *name_args):
        """
        Returns true if named parameter(s) was specified on command line
        """
        return utils.lib.zargs_hasx(self._p, utils.to_bytes(name), *name_args)

    def print_py(self):
        """
        Print an instance of zargs.
        """
        utils.lib.zargs_print(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zargs_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zcert(object):
    """
    work with CURVE security certificates
    """

    def __init__(self):
        """
        Create and initialize a new certificate in memory
        """
        p = utils.lib.zcert_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zcert_destroy_py)

    @staticmethod
    def new_from(public_key, secret_key):
        """
        Accepts public/secret key pair from caller
        """
        return utils.lib.zcert_new_from(public_key, secret_key)

    @staticmethod
    def new_from_txt(public_txt, secret_txt):
        """
        Accepts public/secret key text pair from caller
        """
        return utils.lib.zcert_new_from_txt(utils.to_bytes(public_txt), utils.to_bytes(secret_txt))

    @staticmethod
    def load(filename):
        """
        Load certificate from file
        """
        return utils.lib.zcert_load(utils.to_bytes(filename))

    def public_key(self):
        """
        Return public part of key pair as 32-byte binary string
        """
        return utils.lib.zcert_public_key(self._p)

    def secret_key(self):
        """
        Return secret part of key pair as 32-byte binary string
        """
        return utils.lib.zcert_secret_key(self._p)

    def public_txt(self):
        """
        Return public part of key pair as Z85 armored string
        """
        return utils.lib.zcert_public_txt(self._p)

    def secret_txt(self):
        """
        Return secret part of key pair as Z85 armored string
        """
        return utils.lib.zcert_secret_txt(self._p)

    def set_meta(self, name, format, *format_args):
        """
        Set certificate metadata from formatted string.
        """
        utils.lib.zcert_set_meta(self._p, utils.to_bytes(name), format, *format_args)

    def unset_meta(self, name):
        """
        Unset certificate metadata.
        """
        utils.lib.zcert_unset_meta(self._p, utils.to_bytes(name))

    def meta(self, name):
        """
        Get metadata value from certificate; if the metadata value doesn't
        exist, returns NULL.
        """
        return utils.lib.zcert_meta(self._p, utils.to_bytes(name))

    def meta_keys(self):
        """
        Get list of metadata fields from certificate. Caller is responsible for
        destroying list. Caller should not modify the values of list items.
        """
        return utils.lib.zcert_meta_keys(self._p)

    def save(self, filename):
        """
        Save full certificate (public + secret) to file for persistent storage
        This creates one public file and one secret file (filename + "_secret").
        """
        return utils.lib.zcert_save(self._p, utils.to_bytes(filename))

    def save_public(self, filename):
        """
        Save public certificate only to file for persistent storage
        """
        return utils.lib.zcert_save_public(self._p, utils.to_bytes(filename))

    def save_secret(self, filename):
        """
        Save secret certificate only to file for persistent storage
        """
        return utils.lib.zcert_save_secret(self._p, utils.to_bytes(filename))

    def apply(self, socket):
        """
        Apply certificate to socket, i.e. use for CURVE security on socket.
        If certificate was loaded from public file, the secret key will be
        undefined, and this certificate will not work successfully.
        """
        utils.lib.zcert_apply(self._p, socket._p)

    def dup(self):
        """
        Return copy of certificate; if certificate is NULL or we exhausted
        heap memory, returns NULL.
        """
        return utils.lib.zcert_dup(self._p)

    def eq(self, compare):
        """
        Return true if two certificates have the same keys
        """
        return utils.lib.zcert_eq(self._p, compare._p)

    def print_py(self):
        """
        Print certificate contents to stdout
        """
        utils.lib.zcert_print(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class
        """
        utils.lib.zcert_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zfile(object):
    """
    helper functions for working with files.
    """

    def __init__(self, path, name):
        """
        If file exists, populates properties. CZMQ supports portable symbolic
        links, which are files with the extension ".ln". A symbolic link is a
        text file containing one line, the filename of a target file. Reading
        data from the symbolic link actually reads from the target file. Path
        may be NULL, in which case it is not used.
        """
        p = utils.lib.zfile_new(utils.to_bytes(path), utils.to_bytes(name))
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zfile_destroy_py)

    @staticmethod
    def tmp():
        """
        Create new temporary file for writing via tmpfile. File is automatically
        deleted on destroy
        """
        return utils.lib.zfile_tmp()

    def dup(self):
        """
        Duplicate a file item, returns a newly constructed item. If the file
        is null, or memory was exhausted, returns null.
        """
        return utils.lib.zfile_dup(self._p)

    def filename(self, path):
        """
        Return file name, remove path if provided
        """
        return utils.lib.zfile_filename(self._p, utils.to_bytes(path))

    def restat(self):
        """
        Refresh file properties from disk; this is not done automatically
        on access methods, otherwise it is not possible to compare directory
        snapshots.
        """
        utils.lib.zfile_restat(self._p)

    def modified(self):
        """
        Return when the file was last modified. If you want this to reflect the
        current situation, call zfile_restat before checking this property.
        """
        return utils.lib.zfile_modified(self._p)

    def cursize(self):
        """
        Return the last-known size of the file. If you want this to reflect the
        current situation, call zfile_restat before checking this property.
        """
        return utils.lib.zfile_cursize(self._p)

    def is_directory(self):
        """
        Return true if the file is a directory. If you want this to reflect
        any external changes, call zfile_restat before checking this property.
        """
        return utils.lib.zfile_is_directory(self._p)

    def is_regular(self):
        """
        Return true if the file is a regular file. If you want this to reflect
        any external changes, call zfile_restat before checking this property.
        """
        return utils.lib.zfile_is_regular(self._p)

    def is_readable(self):
        """
        Return true if the file is readable by this process. If you want this to
        reflect any external changes, call zfile_restat before checking this
        property.
        """
        return utils.lib.zfile_is_readable(self._p)

    def is_writeable(self):
        """
        Return true if the file is writeable by this process. If you want this
        to reflect any external changes, call zfile_restat before checking this
        property.
        """
        return utils.lib.zfile_is_writeable(self._p)

    def is_stable(self):
        """
        Check if file has stopped changing and can be safely processed.
        Updates the file statistics from disk at every call.
        """
        return utils.lib.zfile_is_stable(self._p)

    def has_changed(self):
        """
        Return true if the file was changed on disk since the zfile_t object
        was created, or the last zfile_restat() call made on it.
        """
        return utils.lib.zfile_has_changed(self._p)

    def remove(self):
        """
        Remove the file from disk
        """
        utils.lib.zfile_remove(self._p)

    def input(self):
        """
        Open file for reading
        Returns 0 if OK, -1 if not found or not accessible
        """
        return utils.lib.zfile_input(self._p)

    def output(self):
        """
        Open file for writing, creating directory if needed
        File is created if necessary; chunks can be written to file at any
        location. Returns 0 if OK, -1 if error.
        """
        return utils.lib.zfile_output(self._p)

    def read(self, bytes, offset):
        """
        Read chunk from file at specified position. If this was the last chunk,
        sets the eof property. Returns a null chunk in case of error.
        """
        return utils.lib.zfile_read(self._p, bytes, offset)

    def eof(self):
        """
        Returns true if zfile_read() just read the last chunk in the file.
        """
        return utils.lib.zfile_eof(self._p)

    def write(self, chunk, offset):
        """
        Write chunk to file at specified position
        Return 0 if OK, else -1
        """
        return utils.lib.zfile_write(self._p, chunk._p, offset)

    def readln(self):
        """
        Read next line of text from file. Returns a pointer to the text line,
        or NULL if there was nothing more to read from the file.
        """
        return utils.lib.zfile_readln(self._p)

    def close(self):
        """
        Close file, if open
        """
        utils.lib.zfile_close(self._p)

    def handle(self):
        """
        Return file handle, if opened
        """
        return utils.lib.zfile_handle(self._p)

    def digest(self):
        """
        Calculate SHA1 digest for file, using zdigest class.
        """
        return utils.lib.zfile_digest(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zfile_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################

from __future__ import print_function
import os
import re
import sys

import subprocess
def pkgconfig_installed ():
   try:
        subprocess.check_output (["pkg-config", "--version"])
        return True
   except subprocess.CalledProcessError:
        return False

def pkgconfig_kwargs (libs):
    """If pkg-config is available, then return kwargs for set_source based on pkg-config output

    It setup include_dirs, library_dirs, libraries and define_macros
    """

    # make API great again!
    if isinstance (libs, (str, bytes)):
        libs = (libs, )

    # drop starting -I -L -l from cflags
    def dropILl (string):
        def _dropILl (string):
            if string.startswith (b"-I") or string.startswith (b"-L") or string.startswith (b"-l"):
                return string [2:]
        return [_dropILl (x) for x in string.split ()]

    # convert -Dfoo=bar to list of tuples [("foo", "bar")] expected by cffi
    def macros (string):
        def _macros (string):
            return tuple (string [2:].split (b'=', 2))
        return [_macros (x) for x in string.split () if x.startswith (b"-D")]

    # pkg-config call
    def pc (libname, *args):
        a = ["pkg-config", "--print-errors"]
        a.extend (args)
        a.append (libname)
        return subprocess.check_output (a)

    # return kwargs for given libname
    def kwargs (libname):
        return {
                "include_dirs" : dropILl (pc (libname, "--cflags-only-I")),
                "library_dirs" : dropILl (pc (libname, "--libs-only-L")),
                "libraries" : dropILl (pc (libname, "--libs-only-l")),
                "define_macros" : macros (pc (libname, "--cflags")),
                }

    # merge all arguments together
    ret = {}
    for libname in libs:
        foo = kwargs (libname)
        for key, value in foo.items ():
            if key not in ret:
                ret [key] = value
            else:
                ret [key].extend (value)

    # Python3 and strict unicode
    for key, value in ret.items ():
        if isinstance (value, bytes):
            ret [key] = value.decode ("utf-8")
        elif isinstance (value, list):
            if len (value) == 0:
                continue
            if isinstance (value[0], tuple):
                ret [key] = [(v[0].decode ("utf-8"), v[1].decode ("utf-8")) for v in value]
            else:
                ret [key] = [v.decode ("utf-8") for v in value]

    return ret

if not pkgconfig_installed ():
    print ("ERROR: build without pkg-config not supported", file=sys.stderr)
    sys.exit (1)

kwargs = pkgconfig_kwargs ([
    "libzmq",
    "uuid",
    "libsystemd",
    "liblz4",
    "libcurl",
    "nss",
    "libmicrohttpd",
    "libczmq"
])
import cffi
# can't import does not work, read and exec manually
with open (os.path.join (
    os.path.dirname (__file__),
    "cdefs.py"), 'r') as fp:
    cdefs_py = fp.read()
gl = {}
exec (cdefs_py, gl)
czmq_cdefs = gl ["czmq_cdefs"]

ffibuilder = cffi.FFI ()
ffibuilder.set_source ("czmq_cffi.native", "#include <czmq.h>", **kwargs)

# Custom setup for czmq
for item in czmq_cdefs:
    ffibuilder.cdef(item)

ffidestructorbuilder = cffi.FFI ()
ffidestructorbuilder.cdef('''
void
   zactor_destroy_py (void *self);

void
   zargs_destroy_py (void *self);

void
   zarmour_destroy_py (void *self);

void
   zcert_destroy_py (void *self);

void
   zcertstore_destroy_py (void *self);

void
   zchunk_destroy_py (void *self);

void
   zconfig_destroy_py (void *self);

void
   zdigest_destroy_py (void *self);

void
   zdir_destroy_py (void *self);

void
   zdir_patch_destroy_py (void *self);

void
   zfile_destroy_py (void *self);

void
   zframe_destroy_py (void *self);

void
   zhash_destroy_py (void *self);

void
   zhashx_destroy_py (void *self);

void
   ziflist_destroy_py (void *self);

void
   zlist_destroy_py (void *self);

void
   zlistx_destroy_py (void *self);

void
   zloop_destroy_py (void *self);

void
   zmsg_destroy_py (void *self);

void
   zpoller_destroy_py (void *self);

void
   zproc_destroy_py (void *self);

void
   zsock_destroy_py (void *self);

void
   ztimerset_destroy_py (void *self);

void
   ztrie_destroy_py (void *self);

void
   zuuid_destroy_py (void *self);

void
   zhttp_client_destroy_py (void *self);

void
   zhttp_server_destroy_py (void *self);

void
   zhttp_server_options_destroy_py (void *self);

void
   zhttp_request_destroy_py (void *self);

void
   zhttp_response_destroy_py (void *self);

void
   zosc_destroy_py (void *self);

''')

ffidestructorbuilder.set_source ("czmq_cffi.destructors", '''
#include <czmq.h>
void
zactor_destroy_py (void *self)
{
   zactor_destroy ((zactor_t **) &self);
}

void
zargs_destroy_py (void *self)
{
   zargs_destroy ((zargs_t **) &self);
}

void
zarmour_destroy_py (void *self)
{
   zarmour_destroy ((zarmour_t **) &self);
}

void
zcert_destroy_py (void *self)
{
   zcert_destroy ((zcert_t **) &self);
}

void
zcertstore_destroy_py (void *self)
{
   zcertstore_destroy ((zcertstore_t **) &self);
}

void
zchunk_destroy_py (void *self)
{
   zchunk_destroy ((zchunk_t **) &self);
}

void
zconfig_destroy_py (void *self)
{
   zconfig_destroy ((zconfig_t **) &self);
}

void
zdigest_destroy_py (void *self)
{
   zdigest_destroy ((zdigest_t **) &self);
}

void
zdir_destroy_py (void *self)
{
   zdir_destroy ((zdir_t **) &self);
}

void
zdir_patch_destroy_py (void *self)
{
   zdir_patch_destroy ((zdir_patch_t **) &self);
}

void
zfile_destroy_py (void *self)
{
   zfile_destroy ((zfile_t **) &self);
}

void
zframe_destroy_py (void *self)
{
   zframe_destroy ((zframe_t **) &self);
}

void
zhash_destroy_py (void *self)
{
   zhash_destroy ((zhash_t **) &self);
}

void
zhashx_destroy_py (void *self)
{
   zhashx_destroy ((zhashx_t **) &self);
}

void
ziflist_destroy_py (void *self)
{
   ziflist_destroy ((ziflist_t **) &self);
}

void
zlist_destroy_py (void *self)
{
   zlist_destroy ((zlist_t **) &self);
}

void
zlistx_destroy_py (void *self)
{
   zlistx_destroy ((zlistx_t **) &self);
}

void
zloop_destroy_py (void *self)
{
   zloop_destroy ((zloop_t **) &self);
}

void
zmsg_destroy_py (void *self)
{
   zmsg_destroy ((zmsg_t **) &self);
}

void
zpoller_destroy_py (void *self)
{
   zpoller_destroy ((zpoller_t **) &self);
}

void
zproc_destroy_py (void *self)
{
   zproc_destroy ((zproc_t **) &self);
}

void
zsock_destroy_py (void *self)
{
   zsock_destroy ((zsock_t **) &self);
}

void
ztimerset_destroy_py (void *self)
{
   ztimerset_destroy ((ztimerset_t **) &self);
}

void
ztrie_destroy_py (void *self)
{
   ztrie_destroy ((ztrie_t **) &self);
}

void
zuuid_destroy_py (void *self)
{
   zuuid_destroy ((zuuid_t **) &self);
}

void
zhttp_client_destroy_py (void *self)
{
   zhttp_client_destroy ((zhttp_client_t **) &self);
}

void
zhttp_server_destroy_py (void *self)
{
   zhttp_server_destroy ((zhttp_server_t **) &self);
}

void
zhttp_server_options_destroy_py (void *self)
{
   zhttp_server_options_destroy ((zhttp_server_options_t **) &self);
}

void
zhttp_request_destroy_py (void *self)
{
   zhttp_request_destroy ((zhttp_request_t **) &self);
}

void
zhttp_response_destroy_py (void *self)
{
   zhttp_response_destroy ((zhttp_response_t **) &self);
}

void
zosc_destroy_py (void *self)
{
   zosc_destroy ((zosc_t **) &self);
}

''', **kwargs)

if __name__ == "__main__":
    ffibuilder.compile (verbose=True)
    ffidestructorbuilder.compile (verbose=True)



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class ZhttpResponse(object):
    """
    Http response that can be received from zhttp_client or sent to zhttp_server.
Class can be reused between send & recv calls.
Headers and Content is being destroyed after every send call.
    """

    def __init__(self):
        """
        Create a new zhttp_response.
        """
        p = utils.lib.zhttp_response_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zhttp_response_destroy_py)

    def send(self, sock, connection):
        """
        Send a response to a request.
        Returns 0 if successful and -1 otherwise.
        """
        return utils.lib.zhttp_response_send(self._p, sock._p, connection._p)

    def recv(self, client, arg, arg2):
        """
        Receive a response from zhttp_client.
        On success return 0, -1 otherwise.

        Recv returns the two user arguments which was provided with the request.
        The reason for two, is to be able to pass around the server connection when forwarding requests or both a callback function and an argument.
        """
        return utils.lib.zhttp_response_recv(self._p, client._p, arg._p, arg2._p)

    def content_type(self):
        """
        Get the response content type
        """
        return utils.lib.zhttp_response_content_type(self._p)

    def set_content_type(self, value):
        """
        Set the content type of the response.
        """
        utils.lib.zhttp_response_set_content_type(self._p, utils.to_bytes(value))

    def status_code(self):
        """
        Get the status code of the response.
        """
        return utils.lib.zhttp_response_status_code(self._p)

    def set_status_code(self, status_code):
        """
        Set the status code of the response.
        """
        utils.lib.zhttp_response_set_status_code(self._p, status_code)

    def headers(self):
        """
        Get the headers of the response.
        """
        return utils.lib.zhttp_response_headers(self._p)

    def content_length(self):
        """
        Get the content length of the response
        """
        return utils.lib.zhttp_response_content_length(self._p)

    def content(self):
        """
        Get the content of the response.
        """
        return utils.lib.zhttp_response_content(self._p)

    def get_content(self):
        """
        Get the content of the response.
        """
        return utils.lib.zhttp_response_get_content(self._p)

    def set_content(self, content):
        """
        Set the content of the response.
        Content must by dynamically allocated string.
        Takes ownership of the content.
        """
        utils.lib.zhttp_response_set_content(self._p, utils.to_bytes(content))

    def set_content_const(self, content):
        """
        Set the content of the response.
        The content is assumed to be constant-memory and will therefore not be copied or deallocated in any way.
        """
        utils.lib.zhttp_response_set_content_const(self._p, utils.to_bytes(content))

    def reset_content(self):
        """
        Set the content to NULL
        """
        utils.lib.zhttp_response_reset_content(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhttp_response_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zlist(object):
    """
    simple generic list container
    """

    def __init__(self):
        """
        Create a new list container
        """
        p = utils.lib.zlist_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zlist_destroy_py)

    def first(self):
        """
        Return the item at the head of list. If the list is empty, returns NULL.
        Leaves cursor pointing at the head item, or NULL if the list is empty.
        """
        return utils.lib.zlist_first(self._p)

    def next(self):
        """
        Return the next item. If the list is empty, returns NULL. To move to
        the start of the list call zlist_first (). Advances the cursor.
        """
        return utils.lib.zlist_next(self._p)

    def last(self):
        """
        Return the item at the tail of list. If the list is empty, returns NULL.
        Leaves cursor pointing at the tail item, or NULL if the list is empty.
        """
        return utils.lib.zlist_last(self._p)

    def head(self):
        """
        Return first item in the list, or null, leaves the cursor
        """
        return utils.lib.zlist_head(self._p)

    def tail(self):
        """
        Return last item in the list, or null, leaves the cursor
        """
        return utils.lib.zlist_tail(self._p)

    def item(self):
        """
        Return the current item of list. If the list is empty, returns NULL.
        Leaves cursor pointing at the current item, or NULL if the list is empty.
        """
        return utils.lib.zlist_item(self._p)

    def append(self, item):
        """
        Append an item to the end of the list, return 0 if OK or -1 if this
        failed for some reason (out of memory). Note that if a duplicator has
        been set, this method will also duplicate the item.
        """
        return utils.lib.zlist_append(self._p, item._p)

    def push(self, item):
        """
        Push an item to the start of the list, return 0 if OK or -1 if this
        failed for some reason (out of memory). Note that if a duplicator has
        been set, this method will also duplicate the item.
        """
        return utils.lib.zlist_push(self._p, item._p)

    def pop(self):
        """
        Pop the item off the start of the list, if any
        """
        return utils.lib.zlist_pop(self._p)

    def exists(self, item):
        """
        Checks if an item already is present. Uses compare method to determine if
        items are equal. If the compare method is NULL the check will only compare
        pointers. Returns true if item is present else false.
        """
        return utils.lib.zlist_exists(self._p, item._p)

    def remove(self, item):
        """
        Remove the specified item from the list if present
        """
        utils.lib.zlist_remove(self._p, item._p)

    def dup(self):
        """
        Make a copy of list. If the list has autofree set, the copied list will
        duplicate all items, which must be strings. Otherwise, the list will hold
        pointers back to the items in the original list. If list is null, returns
        NULL.
        """
        return utils.lib.zlist_dup(self._p)

    def purge(self):
        """
        Purge all items from list
        """
        utils.lib.zlist_purge(self._p)

    def size(self):
        """
        Return number of items in the list
        """
        return utils.lib.zlist_size(self._p)

    def sort(self, compare):
        """
        Sort the list. If the compare function is null, sorts the list by
        ascending key value using a straight ASCII comparison. If you specify
        a compare function, this decides how items are sorted. The sort is not
        stable, so may reorder items with the same keys. The algorithm used is
        combsort, a compromise between performance and simplicity.
        """
        utils.lib.zlist_sort(self._p, compare)

    def autofree(self):
        """
        Set list for automatic item destruction; item values MUST be strings.
        By default a list item refers to a value held elsewhere. When you set
        this, each time you append or push a list item, zlist will take a copy
        of the string value. Then, when you destroy the list, it will free all
        item values automatically. If you use any other technique to allocate
        list values, you must free them explicitly before destroying the list.
        The usual technique is to pop list items and destroy them, until the
        list is empty.
        """
        utils.lib.zlist_autofree(self._p)

    def comparefn(self, fn):
        """
        Sets a compare function for this list. The function compares two items.
        It returns an integer less than, equal to, or greater than zero if the
        first item is found, respectively, to be less than, to match, or be
        greater than the second item.
        This function is used for sorting, removal and exists checking.
        """
        utils.lib.zlist_comparefn(self._p, fn)

    def freefn(self, item, fn, at_tail):
        """
        Set a free function for the specified list item. When the item is
        destroyed, the free function, if any, is called on that item.
        Use this when list items are dynamically allocated, to ensure that
        you don't have memory leaks. You can pass 'free' or NULL as a free_fn.
        Returns the item, or NULL if there is no such item.
        """
        return utils.lib.zlist_freefn(self._p, item._p, fn, at_tail)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zlist_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from setuptools import setup

setup(
    name = "czmq",
    version = "4.2.1",
    license = "mplv2",
    description = """Python bindings of: the high-level c binding for 0mq""",
    url = "https://github.com/zeromq/czmq",
    packages = ["czmq"],
    install_requires = [
    ],
)
################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zsock(object):
    """
    high-level socket API that hides libzmq contexts and sockets
    """

    def __init__(self, type):
        """
        Create a new socket. Returns the new socket, or NULL if the new socket
        could not be created. Note that the symbol zsock_new (and other
        constructors/destructors for zsock) are redirected to the *_checked
        variant, enabling intelligent socket leak detection. This can have
        performance implications if you use a LOT of sockets. To turn off this
        redirection behaviour, define ZSOCK_NOCHECK.
        """
        p = utils.lib.zsock_new(type)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zsock_destroy_py)

    @staticmethod
    def new_pub(endpoint):
        """
        Create a PUB socket. Default action is bind.
        """
        return utils.lib.zsock_new_pub(utils.to_bytes(endpoint))

    @staticmethod
    def new_sub(endpoint, subscribe):
        """
        Create a SUB socket, and optionally subscribe to some prefix string. Default
        action is connect.
        """
        return utils.lib.zsock_new_sub(utils.to_bytes(endpoint), utils.to_bytes(subscribe))

    @staticmethod
    def new_req(endpoint):
        """
        Create a REQ socket. Default action is connect.
        """
        return utils.lib.zsock_new_req(utils.to_bytes(endpoint))

    @staticmethod
    def new_rep(endpoint):
        """
        Create a REP socket. Default action is bind.
        """
        return utils.lib.zsock_new_rep(utils.to_bytes(endpoint))

    @staticmethod
    def new_dealer(endpoint):
        """
        Create a DEALER socket. Default action is connect.
        """
        return utils.lib.zsock_new_dealer(utils.to_bytes(endpoint))

    @staticmethod
    def new_router(endpoint):
        """
        Create a ROUTER socket. Default action is bind.
        """
        return utils.lib.zsock_new_router(utils.to_bytes(endpoint))

    @staticmethod
    def new_push(endpoint):
        """
        Create a PUSH socket. Default action is connect.
        """
        return utils.lib.zsock_new_push(utils.to_bytes(endpoint))

    @staticmethod
    def new_pull(endpoint):
        """
        Create a PULL socket. Default action is bind.
        """
        return utils.lib.zsock_new_pull(utils.to_bytes(endpoint))

    @staticmethod
    def new_xpub(endpoint):
        """
        Create an XPUB socket. Default action is bind.
        """
        return utils.lib.zsock_new_xpub(utils.to_bytes(endpoint))

    @staticmethod
    def new_xsub(endpoint):
        """
        Create an XSUB socket. Default action is connect.
        """
        return utils.lib.zsock_new_xsub(utils.to_bytes(endpoint))

    @staticmethod
    def new_pair(endpoint):
        """
        Create a PAIR socket. Default action is connect.
        """
        return utils.lib.zsock_new_pair(utils.to_bytes(endpoint))

    @staticmethod
    def new_stream(endpoint):
        """
        Create a STREAM socket. Default action is connect.
        """
        return utils.lib.zsock_new_stream(utils.to_bytes(endpoint))

    @staticmethod
    def new_server(endpoint):
        """
        Create a SERVER socket. Default action is bind.
        """
        return utils.lib.zsock_new_server(utils.to_bytes(endpoint))

    @staticmethod
    def new_client(endpoint):
        """
        Create a CLIENT socket. Default action is connect.
        """
        return utils.lib.zsock_new_client(utils.to_bytes(endpoint))

    @staticmethod
    def new_radio(endpoint):
        """
        Create a RADIO socket. Default action is bind.
        """
        return utils.lib.zsock_new_radio(utils.to_bytes(endpoint))

    @staticmethod
    def new_dish(endpoint):
        """
        Create a DISH socket. Default action is connect.
        """
        return utils.lib.zsock_new_dish(utils.to_bytes(endpoint))

    @staticmethod
    def new_gather(endpoint):
        """
        Create a GATHER socket. Default action is bind.
        """
        return utils.lib.zsock_new_gather(utils.to_bytes(endpoint))

    @staticmethod
    def new_scatter(endpoint):
        """
        Create a SCATTER socket. Default action is connect.
        """
        return utils.lib.zsock_new_scatter(utils.to_bytes(endpoint))

    @staticmethod
    def new_dgram(endpoint):
        """
        Create a DGRAM (UDP) socket. Default action is bind.
        The endpoint is a string consisting of a
        'transport'`://` followed by an 'address'. As this is
        a UDP socket the 'transport' has to be 'udp'. The
        'address' specifies the ip address and port to
        bind to. For example:  udp://127.0.0.1:1234
        Note: To send to an endpoint over UDP you have to
        send a message with the destination endpoint address
        as a first message!
        """
        return utils.lib.zsock_new_dgram(utils.to_bytes(endpoint))

    def bind(self, format, *format_args):
        """
        Bind a socket to a formatted endpoint. For tcp:// endpoints, supports
        ephemeral ports, if you specify the port number as "*". By default
        zsock uses the IANA designated range from C000 (49152) to FFFF (65535).
        To override this range, follow the "*" with "[first-last]". Either or
        both first and last may be empty. To bind to a random port within the
        range, use "!" in place of "*".

        Examples:
            tcp://127.0.0.1:*           bind to first free port from C000 up
            tcp://127.0.0.1:!           bind to random port from C000 to FFFF
            tcp://127.0.0.1:*[60000-]   bind to first free port from 60000 up
            tcp://127.0.0.1:![-60000]   bind to random port from C000 to 60000
            tcp://127.0.0.1:![55000-55999]
                                        bind to random port from 55000 to 55999

        On success, returns the actual port number used, for tcp:// endpoints,
        and 0 for other transports. On failure, returns -1. Note that when using
        ephemeral ports, a port may be reused by different services without
        clients being aware. Protocols that run on ephemeral ports should take
        this into account.
        """
        return utils.lib.zsock_bind(self._p, format, *format_args)

    def endpoint(self):
        """
        Returns last bound endpoint, if any.
        """
        return utils.lib.zsock_endpoint(self._p)

    def unbind(self, format, *format_args):
        """
        Unbind a socket from a formatted endpoint.
        Returns 0 if OK, -1 if the endpoint was invalid or the function
        isn't supported.
        """
        return utils.lib.zsock_unbind(self._p, format, *format_args)

    def connect(self, format, *format_args):
        """
        Connect a socket to a formatted endpoint
        Returns 0 if OK, -1 if the endpoint was invalid.
        """
        return utils.lib.zsock_connect(self._p, format, *format_args)

    def disconnect(self, format, *format_args):
        """
        Disconnect a socket from a formatted endpoint
        Returns 0 if OK, -1 if the endpoint was invalid or the function
        isn't supported.
        """
        return utils.lib.zsock_disconnect(self._p, format, *format_args)

    def attach(self, endpoints, serverish):
        """
        Attach a socket to zero or more endpoints. If endpoints is not null,
        parses as list of ZeroMQ endpoints, separated by commas, and prefixed by
        '@' (to bind the socket) or '>' (to connect the socket). Returns 0 if all
        endpoints were valid, or -1 if there was a syntax error. If the endpoint
        does not start with '@' or '>', the serverish argument defines whether
        it is used to bind (serverish = true) or connect (serverish = false).
        """
        return utils.lib.zsock_attach(self._p, utils.to_bytes(endpoints), serverish)

    def type_str(self):
        """
        Returns socket type as printable constant string.
        """
        return utils.lib.zsock_type_str(self._p)

    def send(self, picture, *picture_args):
        """
        Send a 'picture' message to the socket (or actor). The picture is a
        string that defines the type of each frame. This makes it easy to send
        a complex multiframe message in one call. The picture can contain any
        of these characters, each corresponding to one or two arguments:

            i = int (signed)
            1 = uint8_t
            2 = uint16_t
            4 = uint32_t
            8 = uint64_t
            s = char *
            b = byte *, size_t (2 arguments)
            c = zchunk_t *
            f = zframe_t *
            h = zhashx_t *
            l = zlistx_t * (DRAFT)
            U = zuuid_t *
            p = void * (sends the pointer value, only meaningful over inproc)
            m = zmsg_t * (sends all frames in the zmsg)
            z = sends zero-sized frame (0 arguments)
            u = uint (deprecated)

        Note that s, b, c, and f are encoded the same way and the choice is
        offered as a convenience to the sender, which may or may not already
        have data in a zchunk or zframe. Does not change or take ownership of
        any arguments. Returns 0 if successful, -1 if sending failed for any
        reason.
        """
        return utils.lib.zsock_send(self._p, utils.to_bytes(picture), *picture_args)

    def vsend(self, picture, argptr):
        """
        Send a 'picture' message to the socket (or actor). This is a va_list
        version of zsock_send (), so please consult its documentation for the
        details.
        """
        return utils.lib.zsock_vsend(self._p, utils.to_bytes(picture), argptr._p)

    def recv(self, picture, *picture_args):
        """
        Receive a 'picture' message to the socket (or actor). See zsock_send for
        the format and meaning of the picture. Returns the picture elements into
        a series of pointers as provided by the caller:

            i = int * (stores signed integer)
            4 = uint32_t * (stores 32-bit unsigned integer)
            8 = uint64_t * (stores 64-bit unsigned integer)
            s = char ** (allocates new string)
            b = byte **, size_t * (2 arguments) (allocates memory)
            c = zchunk_t ** (creates zchunk)
            f = zframe_t ** (creates zframe)
            U = zuuid_t * (creates a zuuid with the data)
            h = zhashx_t ** (creates zhashx)
            l = zlistx_t ** (creates zlistx) (DRAFT)
            p = void ** (stores pointer)
            m = zmsg_t ** (creates a zmsg with the remaining frames)
            z = null, asserts empty frame (0 arguments)
            u = uint * (stores unsigned integer, deprecated)

        Note that zsock_recv creates the returned objects, and the caller must
        destroy them when finished with them. The supplied pointers do not need
        to be initialized. Returns 0 if successful, or -1 if it failed to recv
        a message, in which case the pointers are not modified. When message
        frames are truncated (a short message), sets return values to zero/null.
        If an argument pointer is NULL, does not store any value (skips it).
        An 'n' picture matches an empty frame; if the message does not match,
        the method will return -1.
        """
        return utils.lib.zsock_recv(self._p, utils.to_bytes(picture), *picture_args)

    def vrecv(self, picture, argptr):
        """
        Receive a 'picture' message from the socket (or actor). This is a
        va_list version of zsock_recv (), so please consult its documentation
        for the details.
        """
        return utils.lib.zsock_vrecv(self._p, utils.to_bytes(picture), argptr._p)

    def bsend(self, picture, *picture_args):
        """
        Send a binary encoded 'picture' message to the socket (or actor). This
        method is similar to zsock_send, except the arguments are encoded in a
        binary format that is compatible with zproto, and is designed to reduce
        memory allocations. The pattern argument is a string that defines the
        type of each argument. Supports these argument types:

         pattern    C type                  zproto type:
            1       uint8_t                 type = "number" size = "1"
            2       uint16_t                type = "number" size = "2"
            4       uint32_t                type = "number" size = "3"
            8       uint64_t                type = "number" size = "4"
            s       char *, 0-255 chars     type = "string"
            S       char *, 0-2^32-1 chars  type = "longstr"
            c       zchunk_t *              type = "chunk"
            f       zframe_t *              type = "frame"
            u       zuuid_t *               type = "uuid"
            m       zmsg_t *                type = "msg"
            p       void *, sends pointer value, only over inproc

        Does not change or take ownership of any arguments. Returns 0 if
        successful, -1 if sending failed for any reason.
        """
        return utils.lib.zsock_bsend(self._p, utils.to_bytes(picture), *picture_args)

    def brecv(self, picture, *picture_args):
        """
        Receive a binary encoded 'picture' message from the socket (or actor).
        This method is similar to zsock_recv, except the arguments are encoded
        in a binary format that is compatible with zproto, and is designed to
        reduce memory allocations. The pattern argument is a string that defines
        the type of each argument. See zsock_bsend for the supported argument
        types. All arguments must be pointers; this call sets them to point to
        values held on a per-socket basis.
        For types 1, 2, 4 and 8 the caller must allocate the memory itself before
        calling zsock_brecv.
        For types S, the caller must free the value once finished with it, as
        zsock_brecv will allocate the buffer.
        For type s, the caller must not free the value as it is stored in a
        local cache for performance purposes.
        For types c, f, u and m the caller must call the appropriate destructor
        depending on the object as zsock_brecv will create new objects.
        For type p the caller must coordinate with the sender, as it is just a
        pointer value being passed.
        """
        return utils.lib.zsock_brecv(self._p, utils.to_bytes(picture), *picture_args)

    def routing_id(self):
        """
        Return socket routing ID if any. This returns 0 if the socket is not
        of type ZMQ_SERVER or if no request was already received on it.
        """
        return utils.lib.zsock_routing_id(self._p)

    def set_routing_id(self, routing_id):
        """
        Set routing ID on socket. The socket MUST be of type ZMQ_SERVER.
        This will be used when sending messages on the socket via the zsock API.
        """
        utils.lib.zsock_set_routing_id(self._p, routing_id)

    def set_unbounded(self):
        """
        Set socket to use unbounded pipes (HWM=0); use this in cases when you are
        totally certain the message volume can fit in memory. This method works
        across all versions of ZeroMQ. Takes a polymorphic socket reference.
        """
        utils.lib.zsock_set_unbounded(self._p)

    def signal(self, status):
        """
        Send a signal over a socket. A signal is a short message carrying a
        success/failure code (by convention, 0 means OK). Signals are encoded
        to be distinguishable from "normal" messages. Accepts a zsock_t or a
        zactor_t argument, and returns 0 if successful, -1 if the signal could
        not be sent. Takes a polymorphic socket reference.
        """
        return utils.lib.zsock_signal(self._p, status)

    def wait(self):
        """
        Wait on a signal. Use this to coordinate between threads, over pipe
        pairs. Blocks until the signal is received. Returns -1 on error, 0 or
        greater on success. Accepts a zsock_t or a zactor_t as argument.
        Takes a polymorphic socket reference.
        """
        return utils.lib.zsock_wait(self._p)

    def flush(self):
        """
        If there is a partial message still waiting on the socket, remove and
        discard it. This is useful when reading partial messages, to get specific
        message types.
        """
        utils.lib.zsock_flush(self._p)

    def join(self, group):
        """
        Join a group for the RADIO-DISH pattern. Call only on ZMQ_DISH.
        Returns 0 if OK, -1 if failed.
        """
        return utils.lib.zsock_join(self._p, utils.to_bytes(group))

    def leave(self, group):
        """
        Leave a group for the RADIO-DISH pattern. Call only on ZMQ_DISH.
        Returns 0 if OK, -1 if failed.
        """
        return utils.lib.zsock_leave(self._p, utils.to_bytes(group))

    @staticmethod
    def is_py(self):
        """
        Probe the supplied object, and report if it looks like a zsock_t.
        Takes a polymorphic socket reference.
        """
        return utils.lib.zsock_is(self._p)

    @staticmethod
    def resolve(self):
        """
        Probe the supplied reference. If it looks like a zsock_t instance, return
        the underlying libzmq socket handle; else if it looks like a file
        descriptor, return NULL; else if it looks like a libzmq socket handle,
        return the supplied value. Takes a polymorphic socket reference.
        """
        return utils.lib.zsock_resolve(self._p)

    def has_in(self):
        """
        Check whether the socket has available message to read.
        """
        return utils.lib.zsock_has_in(self._p)

    def priority(self):
        """
        Get socket option `priority`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_priority(self._p)

    def set_priority(self, priority):
        """
        Set socket option `priority`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_priority(self._p, priority)

    def reconnect_stop(self):
        """
        Get socket option `reconnect_stop`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_reconnect_stop(self._p)

    def set_reconnect_stop(self, reconnect_stop):
        """
        Set socket option `reconnect_stop`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_reconnect_stop(self._p, reconnect_stop)

    def set_only_first_subscribe(self, only_first_subscribe):
        """
        Set socket option `only_first_subscribe`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_only_first_subscribe(self._p, only_first_subscribe)

    def set_hello_msg(self, hello_msg):
        """
        Set socket option `hello_msg`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_hello_msg(self._p, hello_msg._p)

    def set_disconnect_msg(self, disconnect_msg):
        """
        Set socket option `disconnect_msg`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_disconnect_msg(self._p, disconnect_msg._p)

    def set_wss_trust_system(self, wss_trust_system):
        """
        Set socket option `wss_trust_system`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_wss_trust_system(self._p, wss_trust_system)

    def set_wss_hostname(self, wss_hostname):
        """
        Set socket option `wss_hostname`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_wss_hostname(self._p, utils.to_bytes(wss_hostname))

    def set_wss_trust_pem(self, wss_trust_pem):
        """
        Set socket option `wss_trust_pem`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_wss_trust_pem(self._p, utils.to_bytes(wss_trust_pem))

    def set_wss_cert_pem(self, wss_cert_pem):
        """
        Set socket option `wss_cert_pem`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_wss_cert_pem(self._p, utils.to_bytes(wss_cert_pem))

    def set_wss_key_pem(self, wss_key_pem):
        """
        Set socket option `wss_key_pem`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_wss_key_pem(self._p, utils.to_bytes(wss_key_pem))

    def out_batch_size(self):
        """
        Get socket option `out_batch_size`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_out_batch_size(self._p)

    def set_out_batch_size(self, out_batch_size):
        """
        Set socket option `out_batch_size`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_out_batch_size(self._p, out_batch_size)

    def in_batch_size(self):
        """
        Get socket option `in_batch_size`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_in_batch_size(self._p)

    def set_in_batch_size(self, in_batch_size):
        """
        Set socket option `in_batch_size`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_in_batch_size(self._p, in_batch_size)

    def socks_password(self):
        """
        Get socket option `socks_password`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_socks_password(self._p)

    def set_socks_password(self, socks_password):
        """
        Set socket option `socks_password`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_socks_password(self._p, utils.to_bytes(socks_password))

    def socks_username(self):
        """
        Get socket option `socks_username`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_socks_username(self._p)

    def set_socks_username(self, socks_username):
        """
        Set socket option `socks_username`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_socks_username(self._p, utils.to_bytes(socks_username))

    def set_xpub_manual_last_value(self, xpub_manual_last_value):
        """
        Set socket option `xpub_manual_last_value`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_xpub_manual_last_value(self._p, xpub_manual_last_value)

    def router_notify(self):
        """
        Get socket option `router_notify`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_router_notify(self._p)

    def set_router_notify(self, router_notify):
        """
        Set socket option `router_notify`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_router_notify(self._p, router_notify)

    def multicast_loop(self):
        """
        Get socket option `multicast_loop`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_multicast_loop(self._p)

    def set_multicast_loop(self, multicast_loop):
        """
        Set socket option `multicast_loop`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_multicast_loop(self._p, multicast_loop)

    def metadata(self):
        """
        Get socket option `metadata`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_metadata(self._p)

    def set_metadata(self, metadata):
        """
        Set socket option `metadata`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_metadata(self._p, utils.to_bytes(metadata))

    def loopback_fastpath(self):
        """
        Get socket option `loopback_fastpath`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_loopback_fastpath(self._p)

    def set_loopback_fastpath(self, loopback_fastpath):
        """
        Set socket option `loopback_fastpath`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_loopback_fastpath(self._p, loopback_fastpath)

    def zap_enforce_domain(self):
        """
        Get socket option `zap_enforce_domain`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_zap_enforce_domain(self._p)

    def set_zap_enforce_domain(self, zap_enforce_domain):
        """
        Set socket option `zap_enforce_domain`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_zap_enforce_domain(self._p, zap_enforce_domain)

    def gssapi_principal_nametype(self):
        """
        Get socket option `gssapi_principal_nametype`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_gssapi_principal_nametype(self._p)

    def set_gssapi_principal_nametype(self, gssapi_principal_nametype):
        """
        Set socket option `gssapi_principal_nametype`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_gssapi_principal_nametype(self._p, gssapi_principal_nametype)

    def gssapi_service_principal_nametype(self):
        """
        Get socket option `gssapi_service_principal_nametype`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_gssapi_service_principal_nametype(self._p)

    def set_gssapi_service_principal_nametype(self, gssapi_service_principal_nametype):
        """
        Set socket option `gssapi_service_principal_nametype`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_gssapi_service_principal_nametype(self._p, gssapi_service_principal_nametype)

    def bindtodevice(self):
        """
        Get socket option `bindtodevice`.
        Available from libzmq 4.3.0.
        """
        return utils.lib.zsock_bindtodevice(self._p)

    def set_bindtodevice(self, bindtodevice):
        """
        Set socket option `bindtodevice`.
        Available from libzmq 4.3.0.
        """
        utils.lib.zsock_set_bindtodevice(self._p, utils.to_bytes(bindtodevice))

    def heartbeat_ivl(self):
        """
        Get socket option `heartbeat_ivl`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_heartbeat_ivl(self._p)

    def set_heartbeat_ivl(self, heartbeat_ivl):
        """
        Set socket option `heartbeat_ivl`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_heartbeat_ivl(self._p, heartbeat_ivl)

    def heartbeat_ttl(self):
        """
        Get socket option `heartbeat_ttl`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_heartbeat_ttl(self._p)

    def set_heartbeat_ttl(self, heartbeat_ttl):
        """
        Set socket option `heartbeat_ttl`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_heartbeat_ttl(self._p, heartbeat_ttl)

    def heartbeat_timeout(self):
        """
        Get socket option `heartbeat_timeout`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_heartbeat_timeout(self._p)

    def set_heartbeat_timeout(self, heartbeat_timeout):
        """
        Set socket option `heartbeat_timeout`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_heartbeat_timeout(self._p, heartbeat_timeout)

    def use_fd(self):
        """
        Get socket option `use_fd`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_use_fd(self._p)

    def set_use_fd(self, use_fd):
        """
        Set socket option `use_fd`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_use_fd(self._p, use_fd)

    def set_xpub_manual(self, xpub_manual):
        """
        Set socket option `xpub_manual`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_xpub_manual(self._p, xpub_manual)

    def set_xpub_welcome_msg(self, xpub_welcome_msg):
        """
        Set socket option `xpub_welcome_msg`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_xpub_welcome_msg(self._p, utils.to_bytes(xpub_welcome_msg))

    def set_stream_notify(self, stream_notify):
        """
        Set socket option `stream_notify`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_stream_notify(self._p, stream_notify)

    def invert_matching(self):
        """
        Get socket option `invert_matching`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_invert_matching(self._p)

    def set_invert_matching(self, invert_matching):
        """
        Set socket option `invert_matching`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_invert_matching(self._p, invert_matching)

    def set_xpub_verboser(self, xpub_verboser):
        """
        Set socket option `xpub_verboser`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_xpub_verboser(self._p, xpub_verboser)

    def connect_timeout(self):
        """
        Get socket option `connect_timeout`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_connect_timeout(self._p)

    def set_connect_timeout(self, connect_timeout):
        """
        Set socket option `connect_timeout`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_connect_timeout(self._p, connect_timeout)

    def tcp_maxrt(self):
        """
        Get socket option `tcp_maxrt`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_tcp_maxrt(self._p)

    def set_tcp_maxrt(self, tcp_maxrt):
        """
        Set socket option `tcp_maxrt`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_tcp_maxrt(self._p, tcp_maxrt)

    def thread_safe(self):
        """
        Get socket option `thread_safe`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_thread_safe(self._p)

    def multicast_maxtpdu(self):
        """
        Get socket option `multicast_maxtpdu`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_multicast_maxtpdu(self._p)

    def set_multicast_maxtpdu(self, multicast_maxtpdu):
        """
        Set socket option `multicast_maxtpdu`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_multicast_maxtpdu(self._p, multicast_maxtpdu)

    def vmci_buffer_size(self):
        """
        Get socket option `vmci_buffer_size`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_vmci_buffer_size(self._p)

    def set_vmci_buffer_size(self, vmci_buffer_size):
        """
        Set socket option `vmci_buffer_size`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_vmci_buffer_size(self._p, vmci_buffer_size)

    def vmci_buffer_min_size(self):
        """
        Get socket option `vmci_buffer_min_size`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_vmci_buffer_min_size(self._p)

    def set_vmci_buffer_min_size(self, vmci_buffer_min_size):
        """
        Set socket option `vmci_buffer_min_size`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_vmci_buffer_min_size(self._p, vmci_buffer_min_size)

    def vmci_buffer_max_size(self):
        """
        Get socket option `vmci_buffer_max_size`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_vmci_buffer_max_size(self._p)

    def set_vmci_buffer_max_size(self, vmci_buffer_max_size):
        """
        Set socket option `vmci_buffer_max_size`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_vmci_buffer_max_size(self._p, vmci_buffer_max_size)

    def vmci_connect_timeout(self):
        """
        Get socket option `vmci_connect_timeout`.
        Available from libzmq 4.2.0.
        """
        return utils.lib.zsock_vmci_connect_timeout(self._p)

    def set_vmci_connect_timeout(self, vmci_connect_timeout):
        """
        Set socket option `vmci_connect_timeout`.
        Available from libzmq 4.2.0.
        """
        utils.lib.zsock_set_vmci_connect_timeout(self._p, vmci_connect_timeout)

    def tos(self):
        """
        Get socket option `tos`.
        Available from libzmq 4.1.0.
        """
        return utils.lib.zsock_tos(self._p)

    def set_tos(self, tos):
        """
        Set socket option `tos`.
        Available from libzmq 4.1.0.
        """
        utils.lib.zsock_set_tos(self._p, tos)

    def set_router_handover(self, router_handover):
        """
        Set socket option `router_handover`.
        Available from libzmq 4.1.0.
        """
        utils.lib.zsock_set_router_handover(self._p, router_handover)

    def set_connect_rid(self, connect_rid):
        """
        Set socket option `connect_rid`.
        Available from libzmq 4.1.0.
        """
        utils.lib.zsock_set_connect_rid(self._p, utils.to_bytes(connect_rid))

    def set_connect_rid_bin(self, connect_rid):
        """
        Set socket option `connect_rid` from 32-octet binary
        Available from libzmq 4.1.0.
        """
        utils.lib.zsock_set_connect_rid_bin(self._p, connect_rid)

    def handshake_ivl(self):
        """
        Get socket option `handshake_ivl`.
        Available from libzmq 4.1.0.
        """
        return utils.lib.zsock_handshake_ivl(self._p)

    def set_handshake_ivl(self, handshake_ivl):
        """
        Set socket option `handshake_ivl`.
        Available from libzmq 4.1.0.
        """
        utils.lib.zsock_set_handshake_ivl(self._p, handshake_ivl)

    def socks_proxy(self):
        """
        Get socket option `socks_proxy`.
        Available from libzmq 4.1.0.
        """
        return utils.lib.zsock_socks_proxy(self._p)

    def set_socks_proxy(self, socks_proxy):
        """
        Set socket option `socks_proxy`.
        Available from libzmq 4.1.0.
        """
        utils.lib.zsock_set_socks_proxy(self._p, utils.to_bytes(socks_proxy))

    def set_xpub_nodrop(self, xpub_nodrop):
        """
        Set socket option `xpub_nodrop`.
        Available from libzmq 4.1.0.
        """
        utils.lib.zsock_set_xpub_nodrop(self._p, xpub_nodrop)

    def set_router_mandatory(self, router_mandatory):
        """
        Set socket option `router_mandatory`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_router_mandatory(self._p, router_mandatory)

    def set_probe_router(self, probe_router):
        """
        Set socket option `probe_router`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_probe_router(self._p, probe_router)

    def set_req_relaxed(self, req_relaxed):
        """
        Set socket option `req_relaxed`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_req_relaxed(self._p, req_relaxed)

    def set_req_correlate(self, req_correlate):
        """
        Set socket option `req_correlate`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_req_correlate(self._p, req_correlate)

    def set_conflate(self, conflate):
        """
        Set socket option `conflate`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_conflate(self._p, conflate)

    def zap_domain(self):
        """
        Get socket option `zap_domain`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_zap_domain(self._p)

    def set_zap_domain(self, zap_domain):
        """
        Set socket option `zap_domain`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_zap_domain(self._p, utils.to_bytes(zap_domain))

    def mechanism(self):
        """
        Get socket option `mechanism`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_mechanism(self._p)

    def plain_server(self):
        """
        Get socket option `plain_server`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_plain_server(self._p)

    def set_plain_server(self, plain_server):
        """
        Set socket option `plain_server`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_plain_server(self._p, plain_server)

    def plain_username(self):
        """
        Get socket option `plain_username`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_plain_username(self._p)

    def set_plain_username(self, plain_username):
        """
        Set socket option `plain_username`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_plain_username(self._p, utils.to_bytes(plain_username))

    def plain_password(self):
        """
        Get socket option `plain_password`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_plain_password(self._p)

    def set_plain_password(self, plain_password):
        """
        Set socket option `plain_password`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_plain_password(self._p, utils.to_bytes(plain_password))

    def curve_server(self):
        """
        Get socket option `curve_server`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_curve_server(self._p)

    def set_curve_server(self, curve_server):
        """
        Set socket option `curve_server`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_curve_server(self._p, curve_server)

    def curve_publickey(self):
        """
        Get socket option `curve_publickey`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_curve_publickey(self._p)

    def set_curve_publickey(self, curve_publickey):
        """
        Set socket option `curve_publickey`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_curve_publickey(self._p, utils.to_bytes(curve_publickey))

    def set_curve_publickey_bin(self, curve_publickey):
        """
        Set socket option `curve_publickey` from 32-octet binary
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_curve_publickey_bin(self._p, curve_publickey)

    def curve_secretkey(self):
        """
        Get socket option `curve_secretkey`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_curve_secretkey(self._p)

    def set_curve_secretkey(self, curve_secretkey):
        """
        Set socket option `curve_secretkey`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_curve_secretkey(self._p, utils.to_bytes(curve_secretkey))

    def set_curve_secretkey_bin(self, curve_secretkey):
        """
        Set socket option `curve_secretkey` from 32-octet binary
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_curve_secretkey_bin(self._p, curve_secretkey)

    def curve_serverkey(self):
        """
        Get socket option `curve_serverkey`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_curve_serverkey(self._p)

    def set_curve_serverkey(self, curve_serverkey):
        """
        Set socket option `curve_serverkey`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_curve_serverkey(self._p, utils.to_bytes(curve_serverkey))

    def set_curve_serverkey_bin(self, curve_serverkey):
        """
        Set socket option `curve_serverkey` from 32-octet binary
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_curve_serverkey_bin(self._p, curve_serverkey)

    def gssapi_server(self):
        """
        Get socket option `gssapi_server`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_gssapi_server(self._p)

    def set_gssapi_server(self, gssapi_server):
        """
        Set socket option `gssapi_server`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_gssapi_server(self._p, gssapi_server)

    def gssapi_plaintext(self):
        """
        Get socket option `gssapi_plaintext`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_gssapi_plaintext(self._p)

    def set_gssapi_plaintext(self, gssapi_plaintext):
        """
        Set socket option `gssapi_plaintext`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_gssapi_plaintext(self._p, gssapi_plaintext)

    def gssapi_principal(self):
        """
        Get socket option `gssapi_principal`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_gssapi_principal(self._p)

    def set_gssapi_principal(self, gssapi_principal):
        """
        Set socket option `gssapi_principal`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_gssapi_principal(self._p, utils.to_bytes(gssapi_principal))

    def gssapi_service_principal(self):
        """
        Get socket option `gssapi_service_principal`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_gssapi_service_principal(self._p)

    def set_gssapi_service_principal(self, gssapi_service_principal):
        """
        Set socket option `gssapi_service_principal`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_gssapi_service_principal(self._p, utils.to_bytes(gssapi_service_principal))

    def ipv6(self):
        """
        Get socket option `ipv6`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_ipv6(self._p)

    def set_ipv6(self, ipv6):
        """
        Set socket option `ipv6`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_ipv6(self._p, ipv6)

    def immediate(self):
        """
        Get socket option `immediate`.
        Available from libzmq 4.0.0.
        """
        return utils.lib.zsock_immediate(self._p)

    def set_immediate(self, immediate):
        """
        Set socket option `immediate`.
        Available from libzmq 4.0.0.
        """
        utils.lib.zsock_set_immediate(self._p, immediate)

    def sndhwm(self):
        """
        Get socket option `sndhwm`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_sndhwm(self._p)

    def set_sndhwm(self, sndhwm):
        """
        Set socket option `sndhwm`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_sndhwm(self._p, sndhwm)

    def rcvhwm(self):
        """
        Get socket option `rcvhwm`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_rcvhwm(self._p)

    def set_rcvhwm(self, rcvhwm):
        """
        Set socket option `rcvhwm`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_rcvhwm(self._p, rcvhwm)

    def maxmsgsize(self):
        """
        Get socket option `maxmsgsize`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_maxmsgsize(self._p)

    def set_maxmsgsize(self, maxmsgsize):
        """
        Set socket option `maxmsgsize`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_maxmsgsize(self._p, maxmsgsize)

    def multicast_hops(self):
        """
        Get socket option `multicast_hops`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_multicast_hops(self._p)

    def set_multicast_hops(self, multicast_hops):
        """
        Set socket option `multicast_hops`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_multicast_hops(self._p, multicast_hops)

    def set_xpub_verbose(self, xpub_verbose):
        """
        Set socket option `xpub_verbose`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_xpub_verbose(self._p, xpub_verbose)

    def tcp_keepalive(self):
        """
        Get socket option `tcp_keepalive`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_tcp_keepalive(self._p)

    def set_tcp_keepalive(self, tcp_keepalive):
        """
        Set socket option `tcp_keepalive`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_tcp_keepalive(self._p, tcp_keepalive)

    def tcp_keepalive_idle(self):
        """
        Get socket option `tcp_keepalive_idle`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_tcp_keepalive_idle(self._p)

    def set_tcp_keepalive_idle(self, tcp_keepalive_idle):
        """
        Set socket option `tcp_keepalive_idle`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_tcp_keepalive_idle(self._p, tcp_keepalive_idle)

    def tcp_keepalive_cnt(self):
        """
        Get socket option `tcp_keepalive_cnt`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_tcp_keepalive_cnt(self._p)

    def set_tcp_keepalive_cnt(self, tcp_keepalive_cnt):
        """
        Set socket option `tcp_keepalive_cnt`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_tcp_keepalive_cnt(self._p, tcp_keepalive_cnt)

    def tcp_keepalive_intvl(self):
        """
        Get socket option `tcp_keepalive_intvl`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_tcp_keepalive_intvl(self._p)

    def set_tcp_keepalive_intvl(self, tcp_keepalive_intvl):
        """
        Set socket option `tcp_keepalive_intvl`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_tcp_keepalive_intvl(self._p, tcp_keepalive_intvl)

    def tcp_accept_filter(self):
        """
        Get socket option `tcp_accept_filter`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_tcp_accept_filter(self._p)

    def set_tcp_accept_filter(self, tcp_accept_filter):
        """
        Set socket option `tcp_accept_filter`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_tcp_accept_filter(self._p, utils.to_bytes(tcp_accept_filter))

    def last_endpoint(self):
        """
        Get socket option `last_endpoint`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_last_endpoint(self._p)

    def set_router_raw(self, router_raw):
        """
        Set socket option `router_raw`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_router_raw(self._p, router_raw)

    def ipv4only(self):
        """
        Get socket option `ipv4only`.
        Available from libzmq 3.0.0.
        """
        return utils.lib.zsock_ipv4only(self._p)

    def set_ipv4only(self, ipv4only):
        """
        Set socket option `ipv4only`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_ipv4only(self._p, ipv4only)

    def set_delay_attach_on_connect(self, delay_attach_on_connect):
        """
        Set socket option `delay_attach_on_connect`.
        Available from libzmq 3.0.0.
        """
        utils.lib.zsock_set_delay_attach_on_connect(self._p, delay_attach_on_connect)

    def hwm(self):
        """
        Get socket option `hwm`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        return utils.lib.zsock_hwm(self._p)

    def set_hwm(self, hwm):
        """
        Set socket option `hwm`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        utils.lib.zsock_set_hwm(self._p, hwm)

    def swap(self):
        """
        Get socket option `swap`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        return utils.lib.zsock_swap(self._p)

    def set_swap(self, swap):
        """
        Set socket option `swap`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        utils.lib.zsock_set_swap(self._p, swap)

    def affinity(self):
        """
        Get socket option `affinity`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_affinity(self._p)

    def set_affinity(self, affinity):
        """
        Set socket option `affinity`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_affinity(self._p, affinity)

    def identity(self):
        """
        Get socket option `identity`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_identity(self._p)

    def set_identity(self, identity):
        """
        Set socket option `identity`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_identity(self._p, utils.to_bytes(identity))

    def rate(self):
        """
        Get socket option `rate`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_rate(self._p)

    def set_rate(self, rate):
        """
        Set socket option `rate`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_rate(self._p, rate)

    def recovery_ivl(self):
        """
        Get socket option `recovery_ivl`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_recovery_ivl(self._p)

    def set_recovery_ivl(self, recovery_ivl):
        """
        Set socket option `recovery_ivl`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_recovery_ivl(self._p, recovery_ivl)

    def recovery_ivl_msec(self):
        """
        Get socket option `recovery_ivl_msec`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        return utils.lib.zsock_recovery_ivl_msec(self._p)

    def set_recovery_ivl_msec(self, recovery_ivl_msec):
        """
        Set socket option `recovery_ivl_msec`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        utils.lib.zsock_set_recovery_ivl_msec(self._p, recovery_ivl_msec)

    def mcast_loop(self):
        """
        Get socket option `mcast_loop`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        return utils.lib.zsock_mcast_loop(self._p)

    def set_mcast_loop(self, mcast_loop):
        """
        Set socket option `mcast_loop`.
        Available from libzmq 2.0.0 to 3.0.0.
        """
        utils.lib.zsock_set_mcast_loop(self._p, mcast_loop)

    def rcvtimeo(self):
        """
        Get socket option `rcvtimeo`.
        Available from libzmq 2.2.0.
        """
        return utils.lib.zsock_rcvtimeo(self._p)

    def set_rcvtimeo(self, rcvtimeo):
        """
        Set socket option `rcvtimeo`.
        Available from libzmq 2.2.0.
        """
        utils.lib.zsock_set_rcvtimeo(self._p, rcvtimeo)

    def sndtimeo(self):
        """
        Get socket option `sndtimeo`.
        Available from libzmq 2.2.0.
        """
        return utils.lib.zsock_sndtimeo(self._p)

    def set_sndtimeo(self, sndtimeo):
        """
        Set socket option `sndtimeo`.
        Available from libzmq 2.2.0.
        """
        utils.lib.zsock_set_sndtimeo(self._p, sndtimeo)

    def sndbuf(self):
        """
        Get socket option `sndbuf`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_sndbuf(self._p)

    def set_sndbuf(self, sndbuf):
        """
        Set socket option `sndbuf`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_sndbuf(self._p, sndbuf)

    def rcvbuf(self):
        """
        Get socket option `rcvbuf`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_rcvbuf(self._p)

    def set_rcvbuf(self, rcvbuf):
        """
        Set socket option `rcvbuf`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_rcvbuf(self._p, rcvbuf)

    def linger(self):
        """
        Get socket option `linger`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_linger(self._p)

    def set_linger(self, linger):
        """
        Set socket option `linger`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_linger(self._p, linger)

    def reconnect_ivl(self):
        """
        Get socket option `reconnect_ivl`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_reconnect_ivl(self._p)

    def set_reconnect_ivl(self, reconnect_ivl):
        """
        Set socket option `reconnect_ivl`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_reconnect_ivl(self._p, reconnect_ivl)

    def reconnect_ivl_max(self):
        """
        Get socket option `reconnect_ivl_max`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_reconnect_ivl_max(self._p)

    def set_reconnect_ivl_max(self, reconnect_ivl_max):
        """
        Set socket option `reconnect_ivl_max`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_reconnect_ivl_max(self._p, reconnect_ivl_max)

    def backlog(self):
        """
        Get socket option `backlog`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_backlog(self._p)

    def set_backlog(self, backlog):
        """
        Set socket option `backlog`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_backlog(self._p, backlog)

    def set_subscribe(self, subscribe):
        """
        Set socket option `subscribe`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_subscribe(self._p, utils.to_bytes(subscribe))

    def set_unsubscribe(self, unsubscribe):
        """
        Set socket option `unsubscribe`.
        Available from libzmq 2.0.0.
        """
        utils.lib.zsock_set_unsubscribe(self._p, utils.to_bytes(unsubscribe))

    def type(self):
        """
        Get socket option `type`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_type(self._p)

    def rcvmore(self):
        """
        Get socket option `rcvmore`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_rcvmore(self._p)

    def fd(self):
        """
        Get socket option `fd`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_fd(self._p)

    def events(self):
        """
        Get socket option `events`.
        Available from libzmq 2.0.0.
        """
        return utils.lib.zsock_events(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zsock_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zlistx(object):
    """
    extended generic list container
    """

    def __init__(self):
        """
        Create a new, empty list.
        """
        p = utils.lib.zlistx_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zlistx_destroy_py)

    @staticmethod
    def unpack(frame):
        """
        Unpack binary frame into a new list. Packed data must follow format
        defined by zlistx_pack. List is set to autofree. An empty frame
        unpacks to an empty list.
        """
        return utils.lib.zlistx_unpack(frame._p)

    def add_start(self, item):
        """
        Add an item to the head of the list. Calls the item duplicator, if any,
        on the item. Resets cursor to list head. Returns an item handle on
        success, NULL if memory was exhausted.
        """
        return utils.lib.zlistx_add_start(self._p, item._p)

    def add_end(self, item):
        """
        Add an item to the tail of the list. Calls the item duplicator, if any,
        on the item. Resets cursor to list head. Returns an item handle on
        success, NULL if memory was exhausted.
        """
        return utils.lib.zlistx_add_end(self._p, item._p)

    def size(self):
        """
        Return the number of items in the list
        """
        return utils.lib.zlistx_size(self._p)

    def head(self):
        """
        Return first item in the list, or null, leaves the cursor
        """
        return utils.lib.zlistx_head(self._p)

    def tail(self):
        """
        Return last item in the list, or null, leaves the cursor
        """
        return utils.lib.zlistx_tail(self._p)

    def first(self):
        """
        Return the item at the head of list. If the list is empty, returns NULL.
        Leaves cursor pointing at the head item, or NULL if the list is empty.
        """
        return utils.lib.zlistx_first(self._p)

    def next(self):
        """
        Return the next item. At the end of the list (or in an empty list),
        returns NULL. Use repeated zlistx_next () calls to work through the list
        from zlistx_first (). First time, acts as zlistx_first().
        """
        return utils.lib.zlistx_next(self._p)

    def prev(self):
        """
        Return the previous item. At the start of the list (or in an empty list),
        returns NULL. Use repeated zlistx_prev () calls to work through the list
        backwards from zlistx_last (). First time, acts as zlistx_last().
        """
        return utils.lib.zlistx_prev(self._p)

    def last(self):
        """
        Return the item at the tail of list. If the list is empty, returns NULL.
        Leaves cursor pointing at the tail item, or NULL if the list is empty.
        """
        return utils.lib.zlistx_last(self._p)

    def item(self):
        """
        Returns the value of the item at the cursor, or NULL if the cursor is
        not pointing to an item.
        """
        return utils.lib.zlistx_item(self._p)

    def cursor(self):
        """
        Returns the handle of the item at the cursor, or NULL if the cursor is
        not pointing to an item.
        """
        return utils.lib.zlistx_cursor(self._p)

    @staticmethod
    def handle_item(handle):
        """
        Returns the item associated with the given list handle, or NULL if passed
        in handle is NULL. Asserts that the passed in handle points to a list element.
        """
        return utils.lib.zlistx_handle_item(handle._p)

    def find(self, item):
        """
        Find an item in the list, searching from the start. Uses the item
        comparator, if any, else compares item values directly. Returns the
        item handle found, or NULL. Sets the cursor to the found item, if any.
        """
        return utils.lib.zlistx_find(self._p, item._p)

    def detach(self, handle):
        """
        Detach an item from the list, using its handle. The item is not modified,
        and the caller is responsible for destroying it if necessary. If handle is
        null, detaches the first item on the list. Returns item that was detached,
        or null if none was. If cursor was at item, moves cursor to previous item,
        so you can detach items while iterating forwards through a list.
        """
        return utils.lib.zlistx_detach(self._p, handle._p)

    def detach_cur(self):
        """
        Detach item at the cursor, if any, from the list. The item is not modified,
        and the caller is responsible for destroying it as necessary. Returns item
        that was detached, or null if none was. Moves cursor to previous item, so
        you can detach items while iterating forwards through a list.
        """
        return utils.lib.zlistx_detach_cur(self._p)

    def delete(self, handle):
        """
        Delete an item, using its handle. Calls the item destructor if any is
        set. If handle is null, deletes the first item on the list. Returns 0
        if an item was deleted, -1 if not. If cursor was at item, moves cursor
        to previous item, so you can delete items while iterating forwards
        through a list.
        """
        return utils.lib.zlistx_delete(self._p, handle._p)

    def move_start(self, handle):
        """
        Move an item to the start of the list, via its handle.
        """
        utils.lib.zlistx_move_start(self._p, handle._p)

    def move_end(self, handle):
        """
        Move an item to the end of the list, via its handle.
        """
        utils.lib.zlistx_move_end(self._p, handle._p)

    def purge(self):
        """
        Remove all items from the list, and destroy them if the item destructor
        is set.
        """
        utils.lib.zlistx_purge(self._p)

    def sort(self):
        """
        Sort the list. If an item comparator was set, calls that to compare
        items, otherwise compares on item value. The sort is not stable, so may
        reorder equal items.
        """
        utils.lib.zlistx_sort(self._p)

    def insert(self, item, low_value):
        """
        Create a new node and insert it into a sorted list. Calls the item
        duplicator, if any, on the item. If low_value is true, starts searching
        from the start of the list, otherwise searches from the end. Use the item
        comparator, if any, to find where to place the new node. Returns a handle
        to the new node, or NULL if memory was exhausted. Resets the cursor to the
        list head.
        """
        return utils.lib.zlistx_insert(self._p, item._p, low_value)

    def reorder(self, handle, low_value):
        """
        Move an item, specified by handle, into position in a sorted list. Uses
        the item comparator, if any, to determine the new location. If low_value
        is true, starts searching from the start of the list, otherwise searches
        from the end.
        """
        utils.lib.zlistx_reorder(self._p, handle._p, low_value)

    def dup(self):
        """
        Make a copy of the list; items are duplicated if you set a duplicator
        for the list, otherwise not. Copying a null reference returns a null
        reference.
        """
        return utils.lib.zlistx_dup(self._p)

    def set_destructor(self, destructor):
        """
        Set a user-defined deallocator for list items; by default items are not
        freed when the list is destroyed.
        """
        utils.lib.zlistx_set_destructor(self._p, destructor)

    def set_duplicator(self, duplicator):
        """
        Set a user-defined duplicator for list items; by default items are not
        copied when the list is duplicated.
        """
        utils.lib.zlistx_set_duplicator(self._p, duplicator)

    def set_comparator(self, comparator):
        """
        Set a user-defined comparator for zlistx_find and zlistx_sort; the method
        must return -1, 0, or 1 depending on whether item1 is less than, equal to,
        or greater than, item2.
        """
        utils.lib.zlistx_set_comparator(self._p, comparator)

    def pack(self):
        """
        Serialize list to a binary frame that can be sent in a message.
        The packed format is compatible with the 'strings' type implemented by zproto:

           ; A list of strings
           list            = list-count *longstr
           list-count      = number-4

           ; Strings are always length + text contents
           longstr         = number-4 *VCHAR

           ; Numbers are unsigned integers in network byte order
           number-4        = 4OCTET
        """
        return utils.lib.zlistx_pack(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zlistx_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
